#!/usr/bin/env python
# -*- coding: utf-8 -*-


import unittest
import tempfile
import os.path
import gzip
import stat
import logging

from urllib2 import URLError
from datetime import datetime
from dateutil.tz import tzutc
from dateutil.tz import tzlocal
from datetime import timedelta


"""
test_task
----------------------------------

Tests for `task` module.
"""

import shutil
import signal
from datetime import date
from d3r.celpp import util
from d3r.celpp.util import DownloadError
from d3r.celpp.task import D3RParameters


class TestUtil(unittest.TestCase):
    def setUp(self):
        pass

    def get_total_seconds(self, td):
        return (td.microseconds +
                (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6

    def test_find_latest_year(self):
        temp_dir = tempfile.mkdtemp()
        try:
            try:
                fakefile = os.path.join(temp_dir, 'fakefile')
                open(fakefile, 'a').close()
                util.find_latest_year(fakefile)
                self.fail('Expected exception')
            except Exception:
                pass
            self.assertEqual(util.find_latest_year(temp_dir), None)

            os.mkdir(os.path.join(temp_dir, "foo"))
            os.mkdir(os.path.join(temp_dir, "2014"))
            self.assertEqual(util.find_latest_year(temp_dir),
                             os.path.join(temp_dir, '2014'))

            open(os.path.join(temp_dir, '2016'), 'a').close()
            self.assertEqual(util.find_latest_year(temp_dir),
                             os.path.join(temp_dir, '2014'))

            os.mkdir(os.path.join(temp_dir, '2012'))
            self.assertEqual(util.find_latest_year(temp_dir),
                             os.path.join(temp_dir, '2014'))

            os.mkdir(os.path.join(temp_dir, '2015'))

            self.assertEqual(util.find_latest_year(temp_dir),
                             os.path.join(temp_dir, '2015'))

        finally:
            shutil.rmtree(temp_dir)

    def test_find_latest_weekly_dataset(self):
        temp_dir = tempfile.mkdtemp()

        try:
            self.assertEqual(util.find_latest_weekly_dataset(temp_dir), None)

            os.mkdir(os.path.join(temp_dir, '2015'))

            self.assertEqual(util.find_latest_weekly_dataset(temp_dir), None)

            open(os.path.join(temp_dir, '2015', 'dataset.week.4'), 'a').close()

            self.assertEqual(util.find_latest_weekly_dataset(temp_dir), None)

            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.3'))

            self.assertEqual(util.find_latest_weekly_dataset(temp_dir),
                             os.path.join(temp_dir, '2015',
                                          'dataset.week.3'))

            # added this check to check fix for issue #13
            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.9'))
            self.assertEqual(util.find_latest_weekly_dataset(temp_dir),
                             os.path.join(temp_dir, '2015',
                                          'dataset.week.9'))

            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week'))
            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.'))

            # added this check to check fix for issue #13
            os.mkdir(os.path.join(temp_dir, '2015', 'dataset.week.10'))
            self.assertEqual(util.find_latest_weekly_dataset(temp_dir),
                             os.path.join(temp_dir, '2015',
                                          'dataset.week.10'))

        finally:
            shutil.rmtree(temp_dir)

    def test_get_celpp_week_of_year(self):
        # try passing none
        try:
            util.get_celpp_week_of_year_from_date(None)
            self.fail('Expected exception')
        except Exception:
            pass

        # try 1-1-2016
        celp_week = util.get_celpp_week_of_year_from_date(date(2016, 1, 1))
        self.assertEqual(celp_week[0], 1)
        self.assertEqual(celp_week[1], 2016)

        # try 12-18-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 12, 18))
        self.assertEqual(celp_week[0], 52)
        self.assertEqual(celp_week[1], 2015)

        # try 12-25-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 12, 25))
        self.assertEqual(celp_week[0], 53)
        self.assertEqual(celp_week[1], 2015)

        # try 12-31-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 12, 31))
        self.assertEqual(celp_week[0], 53)
        self.assertEqual(celp_week[1], 2015)

        # try 10-5-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 10, 5))
        self.assertEqual(celp_week[0], 41)
        self.assertEqual(celp_week[1], 2015)

        # try 10-8-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 10, 8))
        self.assertEqual(celp_week[0], 41)
        self.assertEqual(celp_week[1], 2015)

        # try 10-9-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 10, 9))
        self.assertEqual(celp_week[0], 42)
        self.assertEqual(celp_week[1], 2015)

        # try 10-10-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 10, 10))
        self.assertEqual(celp_week[0], 42)
        self.assertEqual(celp_week[1], 2015)

        # try 10-11-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 10, 11))
        self.assertEqual(celp_week[0], 42)
        self.assertEqual(celp_week[1], 2015)

        # try 10-12-2015
        celp_week = util.get_celpp_week_of_year_from_date(date(2015, 10, 12))
        self.assertEqual(celp_week[0], 42)
        self.assertEqual(celp_week[1], 2015)

    def test_create_celpp_week_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:

            try:
                # try both parameters None
                util.create_celpp_week_dir(None, None)
                self.fail('Expected exception')
            except Exception:
                pass

            try:
                # try week tuple None
                util.create_celpp_week_dir(None, temp_dir)
            except Exception:
                pass

            try:
                # try celppdir None
                util.create_celpp_week_dir((1, 2015), None)
            except Exception:
                pass

            # try path already exists as dir
            os.makedirs(os.path.join(temp_dir, '2015', 'dataset.week.1'))
            util.create_celpp_week_dir((1, 2015), temp_dir)

            # try path already exists as file so it will fail
            open(os.path.join(temp_dir, '2015', 'dataset.week.2'), 'a').close()
            try:
                util.create_celpp_week_dir((2, 2015), temp_dir)
                self.fail('Expected OSError')
            except OSError:
                pass

            # try path successful create
            util.create_celpp_week_dir((3, 2015), temp_dir)
            new_dir = os.path.join(temp_dir, '2015', 'dataset.week.3')
            self.assertEquals(os.path.isdir(new_dir), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_download_url_to_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # url is None
            try:
                util.download_url_to_file(None, None, 0, 0)
                self.fail('Expected DownloadError')
            except DownloadError as d:
                self.assertEquals(str(d), 'url is not set')

            # download_path is None
            try:
                util.download_url_to_file('foo', None, 0, 0)
                self.fail('Expected DownloadError')
            except DownloadError as d:
                self.assertEquals(str(d), 'download_path is not set')

            # retry sleep time negative
            try:
                util.download_url_to_file('foo', 'ha', 0, -1)
                self.fail('Expected DownloadError')
            except DownloadError as d:
                self.assertEquals(str(d), 'retry sleep time cannot be ' +
                                  'negative')

            # error download
            try:
                util.download_url_to_file('file://' + temp_dir +
                                          '/haha',
                                          os.path.join(temp_dir, 'boo'), 0, 0)
                self.fail('Expected DownloadError')
            except DownloadError as d:
                self.assertEquals(str(d), 'Unable to download file from ' +
                                  'file://' + temp_dir + '/haha to ' +
                                  os.path.join(temp_dir, 'boo'))

            # successful download retry sleep and num retries both None
            fake_file = os.path.join(temp_dir, 'hello')
            f = open(fake_file, 'w')
            f.write('hi\n')
            f.flush()
            f.close()
            util.download_url_to_file('file://' + fake_file,
                                      os.path.join(temp_dir, 'boo'),
                                      None, None)
            self.assertEquals(os.path.isfile(os.path.join(temp_dir, 'boo')),
                              True)

            # successful download retry sleep and num retries both 0
            util.download_url_to_file('file://' + fake_file,
                                      os.path.join(temp_dir, 'boo2'), 0, 0)
            self.assertEquals(os.path.isfile(os.path.join(temp_dir, 'boo2')),
                              True)

        finally:
            shutil.rmtree(temp_dir)

    def test_gunzip_file(self):
        temp_dir = tempfile.mkdtemp()
        try:

            # try invalid input
            try:
                util.gunzip_file(os.path.join(temp_dir, 'nonexistantfile'),
                                 os.path.join(temp_dir, 'ha'))
                self.fail('Expected IOError')
            except IOError:
                pass

            # try invalid output
            foogz = os.path.join(temp_dir, 'foo.gz')
            f = gzip.open(foogz, 'wb')
            f.write('hello\n')
            f.flush()
            f.close()
            try:

                util.gunzip_file(foogz,
                                 os.path.join(temp_dir, 'ha', 'ha'))
                self.fail('Expected IOError')
            except IOError:
                pass

            # try valid zip file
            util.gunzip_file(foogz,
                             os.path.join(temp_dir, 'foo'))

            f = open(os.path.join(temp_dir, 'foo'), 'r')
            self.assertEqual(f.readline(), 'hello\n')
            f.close()

        finally:
            shutil.rmtree(temp_dir)

    def test_append_string_to_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            try:
                # Try with double None
                util.append_string_to_file(None, None)
                self.fail('Expected TypeError')
            except TypeError:
                pass

            try:
                # Try with value set to None
                util.append_string_to_file(os.path.join(temp_dir, 'foo'),
                                           None)
            except TypeError:
                pass

            # try on non existant file
            util.append_string_to_file(os.path.join(temp_dir, 'non'), 'hi')
            f = open(os.path.join(temp_dir, 'non'))
            self.assertEqual(f.readline(), 'hi')
            f.close()

            # try on empty file
            open(os.path.join(temp_dir, 'empty'), 'a').close()
            util.append_string_to_file(os.path.join(temp_dir, 'empty'), 'hi')
            f = open(os.path.join(temp_dir, 'empty'))
            self.assertEqual(f.readline(), 'hi')
            f.close()

            # try on file with data
            util.append_string_to_file(os.path.join(temp_dir, 'empty'), 'how')
            f = open(os.path.join(temp_dir, 'empty'))
            self.assertEqual(f.readline(), 'hihow')

        finally:
            shutil.rmtree(temp_dir)

    def test_get_file_line_count(self):
        temp_dir = tempfile.mkdtemp()
        try:
            try:
                # Try with double None
                util.get_file_line_count(None)
                self.fail('Expected TypeError')
            except TypeError:
                pass

            try:
                # try on non existant file
                util.get_file_line_count(os.path.join(temp_dir, 'non'))
            except IOError:
                pass

            # try on empty file
            open(os.path.join(temp_dir, 'empty'), 'a').close()
            self.assertEqual(util.get_file_line_count(os.path.join(temp_dir,
                                                                   'empty')),
                             0)

            # try on file with data
            util.append_string_to_file(os.path.join(temp_dir, 'three'),
                                       'h\no\nw\n')
            self.assertEqual(util.get_file_line_count(os.path.join(temp_dir,
                                                                   'three')),
                             3)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_all_celpp_years(self):
        temp_dir = tempfile.mkdtemp()
        try:
            try:
                util.get_all_celpp_years(os.path.join(temp_dir,
                                                      'doesnotexist'))
                self.fail('Expected exception')
            except Exception:
                pass

            # try with empty dir
            ylist = util.get_all_celpp_years(temp_dir)
            self.assertEqual(len(ylist), 0)

            # try on dir with no valid year directories
            os.mkdir(os.path.join(temp_dir, '123'))
            ylist = util.get_all_celpp_years(temp_dir)
            self.assertEqual(len(ylist), 0)

            # try on dir with 1 valid year
            os.mkdir(os.path.join(temp_dir, '2017'))
            ylist = util.get_all_celpp_years(temp_dir)
            self.assertEqual(len(ylist), 1)
            self.assertEqual(ylist[0], '2017')

            # try on dir with multiple valid year directories
            os.mkdir(os.path.join(temp_dir, '2016'))
            ylist = util.get_all_celpp_years(temp_dir)
            self.assertEqual(len(ylist), 2)
            ylist.sort()
            self.assertEqual(ylist[0], '2016')
            self.assertEqual(ylist[1], '2017')

        finally:
            shutil.rmtree(temp_dir)

    def test_get_celpp_year_from_path(self):

        # pass in None
        try:
            util.get_celpp_year_from_path(None)
            self.fail('Expected exception')
        except Exception:
            pass

        # pass in empty string
        self.assertEqual(util.get_celpp_year_from_path(''), '0')

        # pass in white space string
        self.assertEqual(util.get_celpp_year_from_path('  '), '0')

        # pass in path ending with year
        self.assertEqual(util.get_celpp_year_from_path('/home/foo/2013'),
                         '2013')

        # pass in path with invalid subdirectory on it
        self.assertEqual(util.get_celpp_year_from_path('/h/f/2013/cheese'),
                         '0')

        # pass in path with valid dataset week on end
        valid_week = os.path.join('h', '2013', util.DATA_SET_WEEK_PREFIX
                                  + '10')
        self.assertEqual(util.get_celpp_year_from_path(valid_week), '2013')

    def test_get_all_celpp_weeks(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # try on invalid dir
            try:
                util.get_all_celpp_weeks(os.path.join(temp_dir,
                                                      'doesnotexist'))
                self.fail('Expected exception')
            except Exception:
                pass

            # try on empty dir
            week_list = util.get_all_celpp_weeks(temp_dir)
            self.assertEqual(len(week_list), 0)

            # try on dir with no valid weeks
            os.mkdir(os.path.join(temp_dir, 'foo'))
            week_list = util.get_all_celpp_weeks(temp_dir)
            self.assertEqual(len(week_list), 0)

            # try on dir with 1 valid week
            validweek = os.path.join(temp_dir, util.DATA_SET_WEEK_PREFIX
                                     + '12')
            os.mkdir(validweek)
            week_list = util.get_all_celpp_weeks(temp_dir)
            self.assertEqual(len(week_list), 1)
            self.assertEqual(week_list[0], '12')

            # try on dir with 2 valid weeks
            validweek = os.path.join(temp_dir, util.DATA_SET_WEEK_PREFIX
                                     + '51')
            os.mkdir(validweek)
            week_list = util.get_all_celpp_weeks(temp_dir)
            self.assertEqual(len(week_list), 2)
            week_list.sort()
            self.assertEqual(week_list[0], '12')
            self.assertEqual(week_list[1], '51')

        finally:
            shutil.rmtree(temp_dir)

    def test_get_celpp_week_number_from_path(self):
        try:
            util.get_celpp_week_number_from_path(None)
            self.fail('Expected exception')
        except Exception:
            pass

        # try with empty string
        foo = util.get_celpp_week_number_from_path('')
        self.assertEqual(foo, '0')

        # try with path missing dataset.week. prefix
        foo = util.get_celpp_week_number_from_path('/h/b/hi')
        self.assertEqual(foo, '0')

        # try with valid path
        foo = util.get_celpp_week_number_from_path('/h/b/'
                                                   + util.DATA_SET_WEEK_PREFIX
                                                   + '7')
        self.assertEqual(foo, '7')

        foo = util.get_celpp_week_number_from_path('/h/b/'
                                                   + util.DATA_SET_WEEK_PREFIX
                                                   + '12')
        self.assertEqual(foo, '12')

    def test_get_previous_date_of_previous_friday_from_date(self):
        # test None passed in
        try:
            util.get_previous_friday_from_date(None)
            self.fail('Expected exception')
        except Exception:
            pass

        # test non date passed in
        try:
            util.get_previous_friday_from_date('hello')
            self.fail('Expected exception')
        except Exception:
            pass

        # test date on Friday
        dt = datetime(2016, 6, 17, 8, 0, 5)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2016)
        self.assertTrue(newdt.day == 17)
        self.assertTrue(newdt.month == 6)
        self.assertTrue(newdt.hour == 8)
        self.assertTrue(newdt.minute == 0)
        self.assertTrue(newdt.second == 5)

        # test date on Saturday
        dt = datetime(2015, 8, 1, 15, 20, 2)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2015)
        self.assertTrue(newdt.day == 31)
        self.assertTrue(newdt.month == 7)
        self.assertTrue(newdt.hour == 15)
        self.assertTrue(newdt.minute == 20)
        self.assertTrue(newdt.second == 2)

        # test date on Sunday
        dt = datetime(2016, 6, 19, 8, 0, 5)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2016)
        self.assertTrue(newdt.day == 17)
        self.assertTrue(newdt.month == 6)

        # test date on Monday
        dt = datetime(2016, 6, 20, 8, 0, 5)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2016)
        self.assertTrue(newdt.day == 17)
        self.assertTrue(newdt.month == 6)

        # test date on Tuesday
        dt = datetime(2016, 6, 21, 8, 0, 5)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2016)
        self.assertTrue(newdt.day == 17)
        self.assertTrue(newdt.month == 6)

        # test date on Wednesday
        dt = datetime(2016, 6, 22, 8, 0, 5)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2016)
        self.assertTrue(newdt.day == 17)
        self.assertTrue(newdt.month == 6)

        # test date on Thursday
        dt = datetime(2016, 6, 23, 8, 0, 5)
        newdt = util.get_previous_friday_from_date(dt)
        self.assertTrue(newdt.year == 2016)
        self.assertTrue(newdt.day == 17)
        self.assertTrue(newdt.month == 6)

    def test_is_datetime_after_celpp_week_start(self):

        try:
            util.is_datetime_after_celpp_week_start(None)
            self.fail('Expected exception')
        except Exception:
            pass
        try:
            util.is_datetime_after_celpp_week_start('hi')
            self.fail('Expected exception')
        except AssertionError:
            pass

        prev_friday = util.get_previous_friday_from_date(
            datetime.now(tzlocal()))
        oneday = timedelta(days=1)
        sat = prev_friday + oneday
        self.assertTrue(util.is_datetime_after_celpp_week_start(sat))
        thurs = prev_friday - oneday
        self.assertFalse(util.is_datetime_after_celpp_week_start(thurs))

    def test_has_url_been_updated_since_start_of_celpp_week(self):
        temp_dir = tempfile.mkdtemp()
        try:
            try:
                util.has_url_been_updated_since_start_of_celpp_week(None)
                self.fail('Expected exception')
            except Exception:
                pass

            fakefile = os.path.join(temp_dir, 'foo')

            try:
                util.has_url_been_updated_since_start_of_celpp_week('file://' +
                                                                    fakefile)
                self.fail('Expected exception')
            except URLError:
                pass

            f = open(fakefile, 'w')
            f.write('hi\n')
            f.flush()
            f.close()

            prev_friday = util.get_previous_friday_from_date(
                datetime.now(tzlocal()))

            thurs = prev_friday - timedelta(days=1)
            dse = thurs - datetime(1970, 01, 01, tzinfo=tzutc())
            secs_since_epoch = self.get_total_seconds(dse)
            os.utime(fakefile, (secs_since_epoch, secs_since_epoch))
            val = util.has_url_been_updated_since_start_of_celpp_week(
                'file://' + fakefile)
            self.assertEqual(val, False)

            sat = prev_friday + timedelta(days=1)
            dse = sat - datetime(1970, 01, 01, tzinfo=tzutc())
            secs_since_epoch = self.get_total_seconds(dse)
            os.utime(fakefile, (secs_since_epoch, secs_since_epoch))
            val = util.has_url_been_updated_since_start_of_celpp_week(
                'file://' + fakefile)
            self.assertEqual(val, True)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_command_not_set(self):
        ecode, out, err = util.run_external_command(None)
        self.assertEqual(ecode, 256)
        self.assertEqual(out, '')
        self.assertEqual(err, 'Command must be set')

    def test_run_external_command_cmd_does_not_exist(self):
        temp_dir = tempfile.mkdtemp()
        try:
            try:
                noexist = os.path.join(temp_dir, 'noexist')
                ecode, out, err = util.run_external_command(noexist)
                self.fail('Expected OSError')
            except OSError:
                pass

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_success_with_output(self):
        temp_dir = tempfile.mkdtemp()
        try:
            script = os.path.join(temp_dir, 'yo.py')

            # create a small python script that outputs args passed
            # in to standard out, writes error to standard error
            #  and exits with 0 exit code
            f = open(script, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('sys.stdout.write(sys.argv[1])\n')
            f.write('sys.stdout.write(sys.argv[2])\n')
            f.write('sys.stderr.write("error")\n')
            f.write('sys.exit(0)\n')
            f.flush()
            f.close()
            os.chmod(script, stat.S_IRWXU)

            ecode, out, err = util.run_external_command(script + ' hi how')

            self.assertEqual(err, 'error')
            self.assertEqual(out, 'hihow')
            self.assertEqual(ecode, 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_fail_with_output(self):
        temp_dir = tempfile.mkdtemp()
        try:
            script = os.path.join(temp_dir, 'yo.py')

            # create a small python script that outputs args passed
            # in to standard out, writes error to standard error
            #  and exits with 0 exit code
            f = open(script, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('sys.stdout.write(sys.argv[1])\n')
            f.write('sys.stdout.write(sys.argv[2])\n')
            f.write('sys.stderr.write("2error")\n')
            f.write('sys.exit(2)\n')
            f.flush()
            f.close()
            os.chmod(script, stat.S_IRWXU)

            ecode, out, err = util.run_external_command(script + ' hi how')

            self.assertEqual(err, '2error')
            self.assertEqual(out, 'hihow')
            self.assertEqual(ecode, 2)

        finally:
            shutil.rmtree(temp_dir)

    def test_setup_logging(self):
        logger = logging.getLogger('funlogger')
        theargs = D3RParameters()
        theargs.loglevel = 'INFO'
        util.setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.task')
                         .getEffectiveLevel(),
                         logging.INFO)
        self.assertEqual(theargs.numericloglevel, logging.INFO)
        logger.debug('test')

        theargs.loglevel = 'DEBUG'
        util.setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.task')
                         .getEffectiveLevel(),
                         logging.DEBUG)
        self.assertEqual(theargs.numericloglevel, logging.DEBUG)

        theargs.loglevel = 'WARNING'
        util.setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.task')
                         .getEffectiveLevel(),
                         logging.WARNING)
        self.assertEqual(theargs.numericloglevel, logging.WARNING)

        theargs.loglevel = 'ERROR'
        util.setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.task')
                         .getEffectiveLevel(),
                         logging.ERROR)
        self.assertEqual(theargs.numericloglevel, logging.ERROR)

        theargs.loglevel = 'CRITICAL'
        util.setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.task')
                         .getEffectiveLevel(),
                         logging.CRITICAL)
        self.assertEqual(theargs.numericloglevel, logging.CRITICAL)

    def test_run_external_command_where_command_is_none(self):
        ecode, out, err = util.run_external_command_with_timeout(None, None)
        self.assertEqual(ecode, 256)
        self.assertEqual(out, '')
        self.assertEqual(err, 'Command must be set')

    def test_run_external_command_where_tmpdir_is_none_or_not_a_dir(self):
        ecode, out, err = util.run_external_command_with_timeout('foo', None)
        self.assertEqual(ecode, 255)
        self.assertEqual(out, '')
        self.assertEqual(err, 'Tmpdir must be set')

        temp_dir = tempfile.mkdtemp()
        try:
            notdir = os.path.join(temp_dir, 'blah')
            ecode, out, err = util.run_external_command_with_timeout('foo',
                                                                     notdir)
            self.assertEqual(ecode, 254)
            self.assertEqual(out, '')
            self.assertEqual(err, 'Tmpdir must be a directory')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            fakecmd = os.path.join(temp_dir, 'fake.py')
            f = open(fakecmd, 'w')
            f.write('#!/usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('sys.stdout.write("somestdout")\n')
            f.write('sys.stderr.write("somestderr")\n')
            f.write('sys.exit(0)\n')
            f.flush()
            f.close()
            os.chmod(fakecmd, stat.S_IRWXU)

            ecode, out, err = util.\
                run_external_command_with_timeout(fakecmd, temp_dir,
                                                  polling_sleep_time=0.1)
            self.assertEqual(ecode, 0)
            self.assertEqual(out, 'somestdout')
            self.assertEqual(err, 'somestderr')

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_timeout_exceeded(self):
        temp_dir = tempfile.mkdtemp()
        try:
            fakecmd = os.path.join(temp_dir, 'fake.py')
            f = open(fakecmd, 'w')
            f.write('#!/usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('import time\n')
            f.write('sys.stdout.write("somestdout")\n')
            f.write('sys.stderr.write("somestderr")\n')
            f.write('sys.stdout.flush()\n')
            f.write('sys.stderr.flush()\n')
            f.write('time.sleep(120)\n')
            f.write('sys.exit(0)\n')
            f.flush()
            f.close()
            os.chmod(fakecmd, stat.S_IRWXU)

            ecode, out, err = util.\
                run_external_command_with_timeout(fakecmd, temp_dir,
                                                  timeout=2,
                                                  kill_delay=2,
                                                  polling_sleep_time=0.1)
            self.assertEqual(ecode, -signal.SIGTERM)
            self.assertEqual(out, 'somestdout')
            self.assertEqual(err, 'somestderr')

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_timeout_exceeded_ignore_sigterm(self):
        temp_dir = tempfile.mkdtemp()
        try:
            fakecmd = os.path.join(temp_dir, 'fake.py')
            f = open(fakecmd, 'w')
            f.write('#!/usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('import time\n')
            f.write('import signal\n\n')
            f.write('def handler(signum, frame):\n')
            f.write('    sys.stdout.write("caught signal " + str(signum))\n\n')
            f.write('signal.signal(signal.SIGTERM, handler)\n')
            f.write('signal.signal(signal.SIGINT, handler)\n')
            f.write('sys.stdout.write("somestdout")\n')
            f.write('sys.stderr.write("somestderr")\n')
            f.write('sys.stdout.flush()\n')
            f.write('sys.stderr.flush()\n')
            f.write('time.sleep(120)\n')
            f.write('time.sleep(120)\n')
            f.write('sys.exit(0)\n')
            f.flush()
            f.close()
            os.chmod(fakecmd, stat.S_IRWXU)

            ecode, out, err = util.\
                run_external_command_with_timeout(fakecmd, temp_dir,
                                                  timeout=1,
                                                  kill_delay=1,
                                                  polling_sleep_time=0.1)

            self.assertEqual(ecode, -signal.SIGKILL)
            self.assertTrue('somestdout' in out)
            self.assertEqual(err, 'somestderr')

        finally:
            shutil.rmtree(temp_dir)

    def test_run_external_command_fail_no_output(self):
        temp_dir = tempfile.mkdtemp()
        try:
            fakecmd = os.path.join(temp_dir, 'fake.py')
            f = open(fakecmd, 'w')
            f.write('#!/usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('sys.exit(1)\n')
            f.flush()
            f.close()
            os.chmod(fakecmd, stat.S_IRWXU)

            ecode, out, err = util.\
                run_external_command_with_timeout(fakecmd, temp_dir,
                                                  polling_sleep_time=0.1)
            self.assertEqual(ecode, 1)
            self.assertEqual(out, '')
            self.assertEqual(err, '')

        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



__author__ = 'churas'

"""
test_vinadocking
--------------------------------

Tests for `vinadocking` module.
"""

import unittest

from d3r import vinadocking


class TestVinaDocking(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_dock_center_missing_comma(self):
        center = '1,2'
        try:
            vinadocking.dock('ligand', 'protein', center)
        except IndexError as ie:
            self.assertEqual(str(ie), 'list index out of range', str(ie))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import logging
import shutil
import os
import time
import tarfile
from d3r.celpp.filetransfer import WebDavFileTransfer


__author__ = 'j5wagner'



def download_tarball(unpack_dir, config, sleep, max_retry = 50):

    os.chdir(unpack_dir)

    f_f_t_obj = WebDavFileTransfer(config)
    f_f_t_obj.connect()
    retries = 0
    while 1:
        #f_f_t_obj.download_file('/dav/celppweekly/challengedata/latest.txt','./latest.txt')
        #f_f_t_obj.download_file(os.path.join(f_f_t_obj.get_remote_challenge_dir(),'latest.txt'),
        #                        './latest.txt')
        f_f_t_obj.download_file('/dav/challengedata/latest.txt','./latest.txt')
        if os.path.getsize('latest.txt') != 0:
            break
        logging.info('Retry # %i: latest.txt not yet available from challengedata directory' %(retries))
        time.sleep(sleep)
        retries += 1
        if retries == max_retry:
            logging.info('Reached max number of retries (%i). Exiting.' %(max_retry))
            return False
        
    chal_tar_name = open('latest.txt').read().strip()
    #box_chal_tar_name = '/dav/celppweekly/challengedata/' + chal_tar_name
    #box_chal_tar_name = os.path.join(f_f_t_obj.get_remote_challenge_dir(),
    #                                 chal_tar_name)
    box_chal_tar_name = '/dav/challengedata/' + chal_tar_name
    logging.info('Downloaded latest.txt. Challenge package name is %s' %(box_chal_tar_name))
    
    retries = 0
    while 1:
        #f_f_t_obj.download_file(box_chal_tar_name,chal_tar_name)
        f_f_t_obj.download_file(box_chal_tar_name,chal_tar_name)
        if os.path.getsize('latest.txt') != 0:
            break
        logging.info('Retry # %i: %s not yet available from challengedata directory' %(retries, box_chal_tar_name))
        time.sleep(sleep)
        retries += 1
        if retries == max_retry:
            logging.info('Reached max number of retries (%i). Exiting.' %(max_retry))
            return False
        
    return chal_tar_name


def main_get_challenge_data(unpack_dir, ftp_config, local_data_file, sleep):
    abs_orig_dir = os.getcwd()
    abs_unpack_dir = os.path.abspath(unpack_dir)
    if ftp_config is None:
        abs_ftp_config = None
    else:
        abs_ftp_config = os.path.abspath(ftp_config)
    ## Download most recent tarball
    
    if local_data_file is None:
        chal_tar_name = download_tarball(abs_unpack_dir, abs_ftp_config, sleep)
        if chal_tar_name == False:
            logging.info('Unable to download challenge package. Exiting.')
            return False
    else:
        abs_local_data_file = os.path.abspath(local_data_file)
        if not(os.path.exists(abs_local_data_file)):
            logging.info('Specified local data package %s does not exist. Exiting.' %(local_data_file))
            return False
        chal_tar_name = abs_local_data_file

    ## Unpack it in the specified directory
    os.chdir(abs_unpack_dir)
    tarfile_obj = tarfile.open(chal_tar_name, 'r:gz')
    tarfile_obj.extractall()
    tarfile_obj.close()
    return True


if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-u", "--unpackdir", metavar="PATH", help = "Dir where the newest challenge data package should be unpacked")
    parser.add_argument("-f", "--ftpconfig", metavar="PATH", help = "File containing user ftp config information (see included example ftp config for specifics)")
    parser.add_argument("-l", "--localdata", metavar="PATH", help = "Use a local challengedata tarball instead of downloading")
    parser.add_argument("-s", "--sleep", metavar="INT", help = "Number of seconds to wait between checking if challenge package exists", default=3600)

    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level = logging.INFO )
    args = parser.parse_args()
    unpack_dir = args.unpackdir
    ftp_config = args.ftpconfig
    local_data = args.localdata
    sleep = args.sleep
    
    abs_running_dir = os.getcwd()
    log_file_path = os.path.join(abs_running_dir, 'final.log')
    log_file_dest = os.path.join(os.path.abspath(unpack_dir), 'final.log')

    main_get_challenge_data(unpack_dir, ftp_config, local_data, sleep)

    #move the final log file to the result dir
    shutil.move(log_file_path, log_file_dest)



#!/usr/bin/env python

__author__ = 'j5wagner'

import os
import sys
import shutil
import re
import glob
import logging
from d3r.utilities.readers import ReadText

logger = logging.getLogger(__name__)

class Dock(object):
    """Abstract class defining methods for a custom docking solution
    for CELPP
    """

    ## To ensure that the workflow copies over the correct scientific prep
    ## outputs, list the expected filetypes here. This script will look
    ## for files in the target prep directory that follow the naming
    ## convention "<cand category>-<target pdbid>_<cand
    ## pdbid>_prepared.<sci_prepped_prot_suffix>", for example
    ## "LMCSS-5b4t_3w8e_prepared.pdb". It will then pass the appropriate
    ## filenames and prefixes to the technical_prep() and dock()
    ## functions.
    SCI_PREPPED_LIG_SUFFIX = '_prepared.sdf'
    SCI_PREPPED_PROT_SUFFIX = '_prepared.pdb'

    def ligand_technical_prep(self, 
                              sci_prepped_lig, 
                              targ_info_dict={}):
        """
        'Technical preparation' is the step immediate preceding
        docking. During this step, you may perform any file
        conversions or processing that are specific to your docking
        program. Implementation of this function is optional.
        :param sci_prepped_lig: Scientifically prepared ligand file
        :param targ_info_dict: A dictionary of information about this target and the candidates chosen for docking.
        :returns: A list of result files to be copied into the
        subsequent docking folder. The base implementation merely
        returns the input string in a list (ie. [sci_prepped_lig]) 
        """
        return [sci_prepped_lig]
    
    def receptor_technical_prep(self, 
                                sci_prepped_receptor, 
                                pocket_center, 
                                targ_info_dict={}):
        """
        'Technical preparation' is the step immediately preceding
        docking. During this step, you may perform any file
        conversions or processing that are specific to your docking
        program. Implementation of this function is optional.
        :param sci_prepped_receptor: Scientifically prepared receptor file
        :param targ_info_dict: A dictionary of information about this target and the candidates chosen for docking.
        :returns: A list of result files to be copied into the
        subsequent docking folder. This implementation merely
        returns the input string in a list (ie [sci_prepped_receptor])
        """
        return [sci_prepped_receptor]

    def dock(self, 
             tech_prepped_lig_list, 
             tech_prepped_receptor_list, 
             output_receptor_pdb, 
             output_lig_mol,
             targ_info_dict={}):
        """
        This function is the only one which the contestant MUST
        implement.  The dock() step runs the actual docking
        algorithm. Its first two arguments are the return values from
        the technical preparation functions for the ligand and
        receptor. These arguments are lists of file names (strings),
        which can be assumed to be in the current directory. 
        If prepare_ligand() and ligand_technical_prep() are not
        implemented by the contestant, tech_prepped_lig_list will
        contain a single string which names a SMILES file in the
        current directory.
        If prepare_protein() and receptor_technical_prep() are not
        implemented by the contestant, tech_prepped_receptor_list will
        contain a single string which names a PDB file in the current
        directory.
        The outputs from this step must be two files - a pdb with the
        filename specified in the output_receptor_pdb argument, and a
        mol with the filename specified in the output_ligand_mol
        argument.
        :param tech_prepped_lig_list: The list of file names resturned by ligand_technical_prep. These have been copied into the current directory.
        :param tech_prepped_receptor_list: The list of file names resturned by receptor_technical_prep. These have been copied into the current directory.
        :param output_receptor_pdb: The final receptor (after docking) must be converted to pdb format and have exactly this file name.
        :param output_lig mol: The final ligand (after docking) must be converted to mol format and have exactly this file name.
        :param targ_info_dict: A dictionary of information about this target and the candidates chosen for docking.
        :returns: True if docking is successful, False otherwise. Unless overwritten, this implementation always returns False
        """
        return False


    def get_pocket_center(self, target_prep_dir):
        pocket_center_file = os.path.join(target_prep_dir,'center.txt')
        try:
            pocket_center = open(pocket_center_file,"r").readlines()[0]
        except:
            logging.info("Unable to find the center file for this case %s" %(target_prep_dir))
            return False
        pocket_center = pocket_center.split(',')
        if not len(pocket_center) == 3:
            logging.info("Pocket center does not parse into 3 coordinates for case %s. Pocket center is %r" %(target_prep_dir, pocket_center))
            return False

        try:
            pocket_center = [float(i.strip()) for i in pocket_center]
        except:
            logging.info('Error converting pocket center coordinates to float for case %s. Pocket center is %r.' %(target_prep_dir, pocket_center))
            return False
        return pocket_center

    def get_sci_prepped_lig(self, target_prep_dir, sci_prepped_lig_suffix):
        sci_prepped_lig_files = glob.glob('%s/lig_*%s' %(target_prep_dir, sci_prepped_lig_suffix))
        # Ensure that there is one unambiguous ligand for docking
        if len(sci_prepped_lig_files) == 0:
            logging.info('No ligand files found for target prep dir %s.' %(target_prep_dir))
            return False
        if len(sci_prepped_lig_files) > 1:
            logging.info('Multiple ligand files found for target prep dir %s (found ligands %r). The workflow currently should only be sending one ligand. Skipping.' %(target_prep_dir, sci_prepped_lig_files))
            return False
        sci_prepped_lig_file = sci_prepped_lig_files[0]
        return sci_prepped_lig_file

    def parse_lig_filename(self, sci_prepped_lig_file):
        try:
            sci_prepped_lig_basename = os.path.basename(sci_prepped_lig_file)
            lig_re_pattern = 'lig_([a-zA-Z0-9]{3})%s' %(self.SCI_PREPPED_LIG_SUFFIX)
            lig_re_results = re.findall(lig_re_pattern, sci_prepped_lig_basename)
        except:
            logging.info('Error parsing ligand filename %s. Skipping.' %(sci_prepped_lig_file))
            return False
        if len(lig_re_results) != 1:
            logging.info('Unable to parse prepared ligand %s. Regular expression matching yielded %r' %(sci_prepped_lig_file, lig_re_results))
            return False
        lig_prefix = lig_re_results[0]
        return lig_prefix

    def parse_cand_name(self, cand_name):
        try:
            cand_re_pattern = '([a-zA-Z0-9]+)-([a-zA-Z0-9]{4})_([a-zA-Z0-9]{4})%s' %(self.SCI_PREPPED_PROT_SUFFIX)
            cand_re_results = re.findall(cand_re_pattern, cand_name)
        except:
            logging.info('Error parsing prepared protein %s. Skipping' %(cand_name))
            return False
        if len(cand_re_results) != 1:
            logging.info('Unable to parse prepared protein %s. Regular expression matching did not yield one match (yielded %r)' %(cand_name, cand_re_results))
            return False
                #raise Exception('Unable to parse prepared protein %s. Regular expression matching yielded %r' %(cand_name, cand_re_results))
        category, target, cand = cand_re_results[0]
        return category, target, cand



    def run_dock(self, prot_sci_prep_dir, lig_sci_prep_dir, dock_dir):
        #os.chdir(prep_result_dir)
        abs_lig_sci_prep_dir = os.path.abspath(lig_sci_prep_dir)
        abs_prot_sci_prep_dir = os.path.abspath(prot_sci_prep_dir)
        abs_dock_dir = os.path.abspath(dock_dir)

        targ_prot_prep_dirs = glob.glob('%s/????' %(abs_prot_sci_prep_dir))
        #targ_lig_prep_dirs = glob.glob('%s/????' %(abs_lig_prep_dir))
        #prepped_prot_targs = [os.path.basename(i.rstrip('/')) for i in targ_prot_prep_dirs]
        #prepped_lig_targs = [os.path.basename(i.rstrip('/')) for i in targ_lig_prep_dirs]
        
        targ_dic = {}
    
        for targ_prot_prep_dir in targ_prot_prep_dirs:
            targ_name = os.path.basename(targ_prot_prep_dir.rstrip('/'))
            targ_lig_prep_dir = os.path.join(abs_lig_sci_prep_dir, targ_name)
            targ_dock_dir = os.path.join(abs_dock_dir,targ_name)
            #os.mkdir(target_dock_dir)
            logging.info("============= Starting to process target:%s =============" %targ_name)
            targ_dic[targ_name] = {}
            # Until we find a good ligand and cand receptor, mark this as invalid
            targ_dic[targ_name]['valid_targ'] = False
        
            # Get the binding pocket center
            pocket_center =  self.get_pocket_center(targ_prot_prep_dir)
            if pocket_center == False:
                logging.info('Failed to find pocket center file in dirctory %s. Skipping target %s.' %(targ_prot_prep_dir, targ_name))
                continue

            # Get the ligand name in this directory
            sci_prepped_lig_file = self.get_sci_prepped_lig(targ_lig_prep_dir,
                                                            Dock.SCI_PREPPED_LIG_SUFFIX)
            if sci_prepped_lig_file == False:
                logging.info('Unable to find single ligand for target dir %s. Skipping target %s.' %(targ_lig_prep_dir, targ_name))
                continue

            # Process ligand file names
            lig_prefix = self.parse_lig_filename(sci_prepped_lig_file)
            if lig_prefix == False:
                logging.info('Unable to parse ligand filename %s. Skipping target %s.' %(sci_prepped_lig_file, targ_name))
                continue

            # Get the cand protein names in this directory
            potential_cand_proteins = glob.glob('%s/*-????_????%s' %(targ_prot_prep_dir, Dock.SCI_PREPPED_PROT_SUFFIX))

            # Process potential cand protein names
            for potential_cand_protein in potential_cand_proteins:
                potential_cand_basename = os.path.basename(potential_cand_protein)
                category, targ_id, cand_id = self.parse_cand_name(potential_cand_basename)
                targ_txt_file = os.path.join(targ_lig_prep_dir,targ_id + '.txt')
                # If this is the first valid cand for this target, make a new dictionary key and mark the target as valid
                if not('valid_cands' in targ_dic[targ_name].keys()):
                    targ_dic[targ_name]['valid_targ'] = True
                    targ_dic[targ_name]['prot_prep_dir'] = targ_prot_prep_dir
                    targ_dic[targ_name]['lig_prep_dir'] = targ_lig_prep_dir
                    targ_dic[targ_name]['pocket_center'] = pocket_center
                    targ_dic[targ_name]['targ_txt_file'] = targ_txt_file
                    targ_dic[targ_name]['lig_file'] = sci_prepped_lig_file
                    targ_dic[targ_name]['lig_prefix'] = lig_prefix
                    targ_dic[targ_name]['valid_cands'] = []
                targ_dic[targ_name]['valid_cands'].append((potential_cand_protein,
                                                          category,
                                                          targ_id,
                                                          cand_id))
            
            
            os.chdir(abs_dock_dir)
    
        # Print out all of the target/cands we will dock to
        logging.info('targ_dic is: ')
        for targ in targ_dic.keys():
            logging.info('%s: %s' %(targ, targ_dic[targ]))

        # Generate a list of targets for docking
        targ_names = targ_dic.keys()
        targ_names = [i for i in targ_names if targ_dic[i]['valid_targ']==True]
        targ_names.sort()

        # Begin populating the target directory
        os.chdir(abs_dock_dir)
        for targ_name in targ_names:
            pocket_center = targ_dic[targ_name]['pocket_center']
            os.chdir(abs_dock_dir)
            # Make the target directory
            os.mkdir(targ_name)
            os.chdir(targ_name)
            abs_targ_dock_dir = os.getcwd()


            # Copy the targ.txt file in 
            copy_dest = '%s/%s' %(abs_targ_dock_dir, os.path.basename(targ_dic[targ_name]['targ_txt_file']))
            shutil.copyfile(targ_dic[targ_name]['targ_txt_file'], copy_dest)
            # Parse the targ.txt file
            ReadText_obj = ReadText()
            targ_info_dict = ReadText_obj.parse_txt(copy_dest)
            targ_info_dict['pocket_center'] = pocket_center

            #### Run CELPPade technical prep

            ### Ligand technical prep

            ## Ligand technical prep setup
            lig_tech_prep_dir = 'lig_%s_tech_prep' %(targ_dic[targ_name]['lig_prefix'])
            os.mkdir(lig_tech_prep_dir)



            # Copy the sci prepped ligand in
            copy_dest = '%s/%s' %(lig_tech_prep_dir, os.path.basename(targ_dic[targ_name]['lig_file']))
            shutil.copyfile(targ_dic[targ_name]['lig_file'], copy_dest)
        
            lig_base_filename = os.path.basename(targ_dic[targ_name]['lig_file'])
            os.chdir(lig_tech_prep_dir)
        
            ## Call user-defined ligand technical prep
            try:
                tech_prepped_lig_file_list = self.ligand_technical_prep(lig_base_filename, targ_info_dict=targ_info_dict)
            except:
                logging.info(sys.exc_info())
                logging.info('try/except statement caught error in function lig_technical_prep. Skipping target %s.' %(targ_name))

                continue
        
            ## Ensure that ligand technical prep was successful
            # Check for function-reported failure
            if tech_prepped_lig_file_list == False:
                logging.info('Technical ligand preparation failed on %s. Skipping target %s.' %(os.path.abspath(lig_base_filename), targ_name))
                continue

            # Ensure that ligand technical prep returns a list of filenames
            if not(type(tech_prepped_lig_file_list) is list):
                logging.info('Technical ligand preparation for %s did not return a list of filenames. Skipping target %s.' %(os.path.abspath(lig_base_filename), targ_name))
                continue

            # Ensure that all files in list really exist
            for filename in tech_prepped_lig_file_list:
                if not(os.path.exists(filename)):
                    logging.info('Technical ligand preparation for %s returned file list %r, but file %s does not exist. Skipping target %s.' %(os.path.abspath(lig_base_filename), tech_prepped_lig_file_list, filename, targ_name))
                    continue
            

            ## Prepare to copy these files for later
            tech_prepped_lig_file_list = [os.path.abspath(i) for i in tech_prepped_lig_file_list]
            targ_dic[targ_name]['tech_prepped_lig_files'] = tech_prepped_lig_file_list

            logging.info('Technical ligand prep successful for %s. All files exist from returned list %r. ' %(os.path.abspath(lig_base_filename), tech_prepped_lig_file_list))

            ### Candidate tech prep
            for cand_file, category, targ_id, cand_id in targ_dic[targ_name]['valid_cands']:
                os.chdir(abs_targ_dock_dir)
            
                ## Candidate tech prep setup
                cand_tech_prep_dir = '%s_%s_tech_prep/' %(category, cand_id)
                os.mkdir(cand_tech_prep_dir)
                copy_dest = '%s/%s' %(cand_tech_prep_dir, os.path.basename(cand_file))
                shutil.copyfile(cand_file, copy_dest)
                prot_base_filename = os.path.basename(cand_file)
                os.chdir(cand_tech_prep_dir)
            
                ## Call user-defined protein technical prep
                try:
                    tech_prepped_prot_file_list = self.receptor_technical_prep(prot_base_filename, pocket_center, targ_info_dict=targ_info_dict)
                except:
                    logging.info(sys.exc_info())
                    logging.info('try/except statement caught error in function receptor_technical_prep.  Skipping candidate %s for target %s.' %(cand_id, targ_name))
                    continue

                ## Ensure that receptor technical prep was successful
                # Check for function-reported failure
                if tech_prepped_prot_file_list == False:
                    logging.info('Technical protein preparation failed on %s. Skipping candidate %s for target %s.' %(os.path.abspath(prot_base_filename), cand_id, targ_name))
                    continue

                # Ensure that receptor technical prep returns a list of filenames
                if not(type(tech_prepped_prot_file_list) is list):
                    logging.info('Technical protein preparation for %s did not return a list of filenames. Skipping candidate %s for target %s.' %(os.path.abspath(prot_base_filename), cand_id, targ_name))
                    continue

                # Ensure that all files in list really exist
                for filename in tech_prepped_prot_file_list:
                    if not(os.path.exists(filename)):
                        logging.info('Technical protein preparation for %s returned file list %r, but file %s does not exist. Skipping candidate %s for target %s.' %(os.path.abspath(lig_base_filename), tech_prepped_lig_file_list, filename, cand_id, targ_name))
                        continue
            
                ## Prepare to copy these files for docking step
                tech_prepped_prot_file_list = [os.path.abspath(i) for i in tech_prepped_prot_file_list]

                logging.info('Protein technical prep successful for %s. All files exist from returned list %r.'%(cand_file, tech_prepped_prot_file_list))
            
        
                #### Run CELPPade docking

                os.chdir(abs_targ_dock_dir)
                cand_dock_dir = '%s_%s_docking' %(category, cand_id)
                os.mkdir(cand_dock_dir)
                os.chdir(cand_dock_dir)

                ## Prepare expected file names
                output_receptor_pdb = '%s-%s_%s_docked.pdb' %(category,
                                                              targ_id,
                                                              cand_id)
                output_lig_mol = '%s-%s_%s_docked.mol' %(category,
                                                         targ_id,
                                                         cand_id)

                ## Copy in tech prepped files
                for filename in tech_prepped_lig_file_list:
                    file_base_name = os.path.basename(filename)
                    shutil.copyfile(filename,file_base_name)
                for filename in tech_prepped_prot_file_list:
                    file_base_name = os.path.basename(filename)
                    shutil.copyfile(filename,file_base_name)
                
                ## Do the actual docking
                try:
                    dock_results = self.dock(tech_prepped_lig_file_list,
                                             tech_prepped_prot_file_list,
                                             output_receptor_pdb,
                                             output_lig_mol,
                                             targ_info_dict=targ_info_dict)
                except:
                    logging.info(sys.exc_info())
                    logging.info('try/except statement caught error in dock() function. Docking was given '
                                 'inputs tech_prepped_lig_file_list=%r tech_prepped_prot_file_list=%r '
                                 'output_receptor_pdb=%r output_lig_mol=%r. Skipping docking to this '
                                 'candidate.'
                                 %(tech_prepped_lig_file_list,
                                   tech_prepped_prot_file_list,
                                   output_receptor_pdb, output_lig_mol))
                    continue

                ## Check for success
                # Check for self-reported failure
                if dock_results == False:
                    logging.info('Docking returned False given inputs: tech_prepped_lig_file_list=%r   '
                                 'tech_prepped_prot_file_list=%r    output_receptor_pdb=%r     output_lig_mol=%r. '
                                 'Skipping docking to this candidate.' %(tech_prepped_lig_file_list,
                                                                        tech_prepped_prot_file_list,
                                                                        output_receptor_pdb,
                                                                        output_lig_mol))
                    continue
                # Ensure that correct output files exist
                if not(os.path.exists(output_receptor_pdb)) or (os.path.getsize(output_receptor_pdb)==0):
                    logging.info('Docking did not create receptor pdb file %s given inputs:   '
                                 'tech_prepped_lig_file_list=%r   tech_prepped_prot_file_list=%r    '
                                 'output_receptor_pdb=%r     output_lig_mol=%r. Skipping docking '
                                 'to this candidate.' %(output_receptor_pdb,
                                                        tech_prepped_lig_file_list,
                                                        tech_prepped_prot_file_list,
                                                        output_receptor_pdb,
                                                        output_lig_mol))
                    continue
                if not(os.path.exists(output_lig_mol)) or (os.path.getsize(output_lig_mol)==0):
                    logging.info('Docking did not create ligand mol file %s given inputs:   '
                                 'tech_prepped_lig_file_list=%r   tech_prepped_prot_file_list=%r    '
                                 'output_receptor_pdb=%r     output_lig_mol=%r. Skipping docking '
                                 'to this candidate.' %(output_lig_mol,
                                                        tech_prepped_lig_file_list,
                                                        tech_prepped_prot_file_list,
                                                        output_receptor_pdb,
                                                        output_lig_mol))
                    continue
            
                logging.info('Docking was successful for %s. Final receptor and ligand '
                             'files %s and %s exist and are nonzero size.' %(cand_file,
                                                                             output_receptor_pdb,
                                                                             output_lig_mol))

                # Prepare to copy docking results into final result directory
                abs_output_receptor_pdb = os.path.abspath(output_receptor_pdb)
                abs_output_lig_mol = os.path.abspath(output_lig_mol)
            
                # Copy the files one directory up
                os.chdir(abs_targ_dock_dir)
                shutil.copyfile(abs_output_receptor_pdb, output_receptor_pdb)
                shutil.copyfile(abs_output_lig_mol, output_lig_mol)



__author__ = 'robswift'

import os
import sys
from d3r.utilities.writers import WriteLog
from d3r.utilities.writers import WriteText
from d3r.utilities.analysis import InputAnalysis
from d3r.utilities.analysis import OutputAnalysis

def writer(out_dir, filtrate, log_file=False):
    """
    Writes out a txt file describing queries and their corresponding docking hits and writes out a blastnfilter log
    :param out_dir: (string) the absolute path to the directory where the log file will be written
    :param filtrate: a list of d3r.blast.Query objects
    :param log_file: (boolean) indicates whether or not a log file should be written
    """
    if not os.access(out_dir, os.W_OK):
        print "%s is not writeable" % out_dir
        sys.exit(1)

    txt_writer = WriteText(out_dir)
    log_writer = WriteLog(out_dir)
    if not filtrate.triage:
        txt_writer.write_txt(filtrate)
    if log_file:
        log_writer.write_log(filtrate)
    log_writer.close_file()

def input_analysis(out_dir, queries):
    report = InputAnalysis(queries)
    report.print_to_standard_out()
    report.print_to_file(out_dir)


class OutController(object):
    def __init__(self):
        self.report = OutputAnalysis()

    def set_query(self, query):
        if not query.triage:
            self.report.query = query
            self.report.set_target_dict()

    def print_filter_criteria(self, out_dir):
        self.report.print_filter_criteria(out_dir)

    def print_to_file(self, out_dir):
        self.report.print_to_file(out_dir)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'jwagner'


import unittest
import tempfile
import os.path
import shutil
import os
import sys

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.evaluation import EvaluationTask


class TestChainPermuter(unittest.TestCase):
    def setUp(self):
        try:
            import openeye.oechem  # noqa: F401
            sys.stdout.write("openeye.oechem importable.")
            self.has_oechem = True
        except:
            sys.stdout.write("openeye.oechem not importable.")
            self.has_oechem = False

        ret_code = os.system('$SCHRODINGER/run split_structure.py &> '
                             '/dev/null')
        if ret_code == 512:
            sys.stdout.write("Schrodinger commandline available.")
            self.has_schrodinger = True
        else:
            sys.stdout.write("Schrodinger commandline not available.")
            self.has_schrodinger = False

    def test_permute_chains_and_take_min_rmsd(self):
        if not(self.has_oechem) or not (self.has_schrodinger):
            sys.stdout.write("Missing essential component. Skipping test.")
            return
        test_scale = 'github'
        method_list = ['autodockvina']
        try:
            temp_dir = tempfile.mkdtemp()
            for method in method_list:
                params = D3RParameters()
                params.evaluation = 'evaluate.py'
                params.pdbdb = os.path.abspath('tests/celpp/eval_test_data/'
                                               '%s_test_data/mini_pdb/'
                                               % (test_scale))
                docktask = D3RTask(temp_dir, params)
                docktask.set_name(method)
                docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
                docktask.create_dir()
                open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                     'a').close()

                source_dir = os.path.abspath('tests/celpp/eval_test_data/'
                                             '%s_test_data/stage.6.%s/'
                                             % (test_scale, method))
                os.system('cp -r %s/* %s' % (source_dir, docktask.get_dir()))
                task = EvaluationTask(temp_dir, '%s.evaluation' % (method),
                                      docktask, params)
                task.run()
                val = task.get_evaluation_summary()
                self.assertEqual(val, '\nEvaluation of docking\n============='
                                      '========\nTarget_PDBID        LMCSS   '
                                      '  SMCSS     hiResApo  hiResHolo hiTani'
                                      'moto \nNumber_of_cases     0         0'
                                      '         1         0         0         '
                                      '\nAverage                             '
                                      '    6.452                         \nMi'
                                      'nimum                                 6'
                                      '.452                         \nMaximum '
                                      '                                6.452  '
                                      '                       \n5t6d          '
                                      '                          6.452        '
                                      '                 \n\n')
                self.assertEqual(task.get_error(), None)
        finally:
            shutil.rmtree(temp_dir)


if __name__ == '__main__':
    unittest.main()



__author__ = 'robswift'

class MCSS(object):
    """
    Stores a maximum common substructure be
    """
    def __init__(self, reference, rd_mol):
        self.reference = reference      # The name of the reference molecule in the MCSS calculation, e.g. -->
                                        # --> Target.dock[0].resname (string)
        self.test = None                # The name of the test molecule in the MCSS calculation. e.g.
                                        # --> Test.dock[0].resname (string)
        self.rd_mol = rd_mol            # A rd mol object that represents the maximum common substructure between -->
                                        # --> the reference molecule and the molecule that contains the MCSS object.
        self.size = None                # The number of atoms in the MCSS (int)
        self.heavy = None
        self.tanimoto = None

    def set_size(self):
        """
        Determines the size, or atom count, of the MCSS. Note that the mol attribute must be set before the MCSS size
        can be set. If the mol object is not set, the method does nothing. Typical usage example:
                mcss = MCSS()
                mcss.reference = ligand.resname
                mcss.mol = Chem.MolFromSmarts(res.smartsString)
                mcss.set_size()
        """
        if not self.rd_mol:
            pass
        else:
            self.size = len(self.rd_mol.GetAtoms())
    #added by sliu 08/08 to add the info for heavy atoms
    def set_heavy_size(self):
        if not self.rd_mol:
            pass
        else:
            self.heavy = self.rd_mol.GetNumHeavyAtoms()



#!/usr/bin/env python

__author__ = 'j5wagner'

import commands
import sys
import os
import glob
import logging
import time
import re 
import shutil 
from d3r.utilities.challenge_data import ChallengeData
from d3r.utilities.readers import ReadText


logger = logging.getLogger(__name__)



class ProteinPrep(object):
    """Abstract class defining methods for a custom protein and ligand preparation solution
    for CELPP
    """



    # This prep script will be required to output files with the appropriate suffixes
    OUTPUT_PROTEIN_SUFFIX = '.pdb'

    def receptor_scientific_prep(self, protein_file, prepared_protein_file, targ_info_dict={}):
        """Does not do any scientific preparation - Passes protein forward without any processing 
        """
        
        # Poor man's receptor splitting - Remove all HETATM and
        # CONECT lines
        #data = open(protein_file).readlines()
        #with open(prepared_protein_file,'wb') as of:
        #    for line in data:
        #        if (line[:6] == 'HETATM') or (line[:6] == 'CONECT'):
        #            continue
        #        else:
        #            of.write(line)
        
        shutil.copyfile(protein_file, prepared_protein_file)
        return True





    def run_scientific_protein_prep(self, challenge_data_path, pdb_protein_path, working_folder):
        abs_challenge_data_path = os.path.abspath(challenge_data_path)
        chal_data_obj = ChallengeData(abs_challenge_data_path)
        if not(chal_data_obj.is_valid_for_celpp()):
            logging.info('%s is not a valid CELPP challenge data directory. Unable to run protein prep.' %(abs_challenge_data_path))
            return False
        week_chal_data_dict = chal_data_obj.get_targets()
        week_name = week_chal_data_dict.keys()[0]
        abs_week_path = os.path.join(abs_challenge_data_path, week_name)
        pot_target_dirs = week_chal_data_dict[week_name]
        os.chdir(working_folder)
        current_dir_layer_1 = os.getcwd()

        ## Get all potential target directories and candidates within
        valid_candidates = {}


        # Ensure that the chalengedata targets are valid and copy in files
        for pot_target_dir in pot_target_dirs:
            os.chdir(current_dir_layer_1)
            pot_target_id = os.path.basename(pot_target_dir.strip('/'))
            # Does it look like a pdb id?
            if len(pot_target_id) != 4:
                logging.info('Filtering potential target directories: %s is not 4 characters long. Skipping' %(pot_target_id))
                continue

            os.mkdir(pot_target_id)

            valid_candidates[pot_target_id] = []
            target_dir_path = os.path.join(abs_week_path, pot_target_id)

            # Copy in <targ id>.txt file
            targ_info_basename = pot_target_id+'.txt'
            targ_info_file = os.path.join(target_dir_path, targ_info_basename)
            targ_info_dest = os.path.join(pot_target_id, targ_info_basename)
            shutil.copyfile(targ_info_file, targ_info_dest)
            
            # Copy in center.txt file
            center_file = os.path.join(target_dir_path,'center.txt')
            #center_file_basename = os.path.basename(center_file)
            center_file_dest = os.path.join(pot_target_id, 'center.txt')
            shutil.copyfile(center_file, center_file_dest)


            # Copy in each valid candidate
            for candidate_file in glob.glob('%s/*-%s_*.pdb' %(target_dir_path, pot_target_id)):
                # The LMCSS ligand will be in a pdb file called something like celpp_week19_2016/1fcz/LMCSS-1fcz_1fcz-156-lig.pdb
                # We want to make sure we don't treat this like a receptor
                if 'lig.pdb' in candidate_file:
                    continue
                candidate_file_basename = os.path.basename(candidate_file)
                candidate_file_dest = os.path.join(pot_target_id,candidate_file_basename)
                shutil.copyfile(candidate_file, candidate_file_dest)
                candidate_local_file = os.path.basename(candidate_file)
                valid_candidates[pot_target_id].append(candidate_local_file)

        for target_id in valid_candidates.keys():
            os.chdir(current_dir_layer_1)
            os.chdir(target_id)
            current_dir_layer_2 = os.getcwd()          
            ReadText_obj = ReadText()
            targ_info_dict = ReadText_obj.parse_txt(target_id + '.txt')
            

            for candidate_filename in valid_candidates[target_id]:
                os.chdir(current_dir_layer_2)
                ## Parse the candidate name 
                ## Get the method type, target, and candidate info from the filename
                # for example, this will parse 'hiResApo-5hib_2eb2.pdb' into [('hiResApo', '5hib', '2eb2')]

                parsed_name = re.findall('([a-zA-Z0-9]+)-([a-zA-Z0-9]+)_([a-zA-Z0-9]+)-?([a-zA-Z0-9]*).pdb', candidate_filename)
                if len(parsed_name) != 1:
                    logging.info('Failed to parse docked structure name "%s". Parsing yielded %r' %(candidate_filename, parsed_name))
                    continue
                candidate_structure_type = parsed_name[0][0]
                candidate_structure_target = parsed_name[0][1]
                candidate_structure_candidate = parsed_name[0][2]
                candidate_structure_ligand = parsed_name[0][2]

                
                candidate_prefix = '%s-%s_%s' %(candidate_structure_type,
                                                candidate_structure_target,
                                                candidate_structure_candidate)          
                # Make candidate prep directory
                os.mkdir(candidate_prefix)
                # Copy in raw candidate file
                candidate_copy_origin = candidate_filename
                candidate_copy_dest = os.path.join(candidate_prefix, candidate_filename)
                shutil.copyfile(candidate_copy_origin, candidate_copy_dest)
                # Copy in center file
                center_copy_origin = 'center.txt'
                center_copy_dest = os.path.join(candidate_prefix, 'center.txt')
                shutil.copyfile(center_copy_origin, center_copy_dest)
                # Move into candidate prep directory
                os.chdir(candidate_prefix)

                # Run prep
                prepared_protein_file = "%s_prepared%s" %(candidate_prefix, ProteinPrep.OUTPUT_PROTEIN_SUFFIX)

                try:
                    preparation_result = self.receptor_scientific_prep(candidate_filename, 
                                                                       prepared_protein_file,
                                                                       targ_info_dict=targ_info_dict)
                except:
                    logging.info(sys.exc_info())
                    logging.info('try/except statement caught error in scientific protein prep. Skipping candidate %s' %(candidate_prefix))
                    continue

                if preparation_result == False:
                    logging.info("Unable to prepare this protein:%s"%(candidate_filename))
                    continue
                if not(os.path.exists(prepared_protein_file)):
                    logging.info('Expected output file %s does not exist after protein preparation. Assuming that protein prep failed. Skipping candidate %s' %(prepared_protein_file, candidate_prefix))
                    continue
                if os.path.getsize(prepared_protein_file)==0:
                    logging.info('Expected output file %s has size 0. Assuming that protein prep failed. Skipping candidate %s' %(prepared_protein_file, candidate_prefix))
                    continue

                prepared_receptor_origin = prepared_protein_file
                prepared_receptor_dest = os.path.join(current_dir_layer_2, prepared_protein_file)
                shutil.copyfile(prepared_receptor_origin, prepared_receptor_dest)          
                
                logging.info("Successfully prepared this protein:%s"%(prepared_protein_file))


            os.chdir(current_dir_layer_1)




#!/usr/bin/env python
__author__ = 'sliu'
#from stage 3 blastnfilter result generating the stage 4 dataset to release to the participant
import os
import sys
import glob
import logging
from rdkit import Chem
from rdkit.Chem import AllChem as allChem
import commands
from Bio import PDB

def split_chain(pdb_filename, out_path, pdb_id, chain_letters):
    parser = PDB.PDBParser()
    writer = PDB.PDBIO()
    chain_letters = [chain.upper() for chain in chain_letters]
    struct = parser.get_structure (pdb_id, pdb_filename)
    writer.set_structure(struct)
    writer.save(out_path, select=SelectChains(chain_letters))

class SelectChains(PDB.Select):
    """ Only accept the specified chains when saving. """
    def __init__(self, chain_letters):
        self.chain_letters = chain_letters

    def accept_chain(self, chain):
        return (chain.get_id() in self.chain_letters)

def parse_txt (txt_filename):
    #parsing the result txt file and give back the

    #1, inchi string for the ligand 
    #2, LMCSS, SMCSS, hiResHolo, hiResApo structure ID
    info_dic = {}
    target_name_txt = os.path.basename(txt_filename).split(".")[0]
    txt_file = open(txt_filename, "r")
    lines = txt_file.readlines()
    txt_file.close()
    for line in lines:
        info_title = line.split(",")[0]
        info_value_1 = line.split(",")[1]
        info_value_3 = False
        if line.split(",")[0] == "inchi":
            info_title = line.split(", InChI=")[0]
            info_value_1 = line.split(", InChI=")[1].split("\n")[0]
        elif line.split(",")[0] in ["LMCSS", "SMCSS", "hiResHolo", "hiTanimoto"]:
            info_title = line.split(",")[0]
            info_value_1 = line.split(",")[1].strip()
            info_value_2 = line.split(",")[2].strip()
            info_value_3 = line.split(",")[3].split(":")[1].strip()
        else:
            try:
                info_value_2 = line.split(",")[2].split("\n")[0].strip()
                info_title = line.split(",")[0]
                info_value_1 = line.split(",")[1].strip()
            except:
                info_value_2 = False
                info_title = line.split(",")[0]
                info_value_1 = line.split(",")[1].split("\n")[0].strip()
                pass
        if not info_value_2:                                  
            #just use the first item if there are multiple entries (use the highest resolution one)
            if info_title not in info_dic:                    
                info_dic[info_title] = info_value_1           
        if info_value_3:                                                 
            if info_title not in info_dic:                    
                info_dic[info_title] = (info_value_1,info_value_2, info_value_3)           
        else:
            if info_title not in info_dic:
                info_dic[info_title] = (info_value_1,info_value_2) 
    #check if the query protein id is identical with the filename
    if not info_dic["query"] == target_name_txt:
        logging.info("The query pdb ID:%s is different from the filename: %s, need to check"%(info_dic["query"], target_name_txt)) 
    return info_dic  


def align_proteins (target_protein, pre_prepare_protein, post_prepare_protein):
    commands.getoutput("$SCHRODINGER/utilities/structalign %s %s"%(target_protein, pre_prepare_protein))
    rotated_protein = "rot-" + pre_prepare_protein
    if os.path.isfile(rotated_protein):
        commands.getoutput("mv %s %s"%(rotated_protein, post_prepare_protein))
        return True
    else:
        return False

def generate_ligand (inchi, ligand_title, ):
    valid_inchi = "InChI="+inchi
    rd_mol = Chem.MolFromInchi(format(valid_inchi), removeHs=False, sanitize=False, treatWarningAsError=True)
    smiles = Chem.MolToSmiles(rd_mol, isomericSmiles=True)
    smile_filename = "lig_" + ligand_title + ".smi"
    smile_file = open(smile_filename, "w")
    smile_file.writelines(smiles)
    inchi_filename = "lig_" + ligand_title + ".inchi"
    inchi_file = open(inchi_filename, "w")
    inchi_file.writelines(valid_inchi)
    mol_filename = "lig_" + ligand_title + ".mol"
    #rd_mol_H = Chem.AddHs(rd_mol)
    #allChem.EmbedMolecule(rd_mol_H)
    #allChem.UFFOptimizeMolecule(rd_mol_H)
    allChem.Compute2DCoords(rd_mol)
    Chem.MolToMolFile (rd_mol, mol_filename, includeStereo=True)

def pull_ligand_out (proteinfile, ligname, ligandfile):
    p_xyz_lines = open(proteinfile,"r").readlines()
    multi_ligand = False
    l_xyz_lines = []
    atom_list = []
    for p_xyz_line in p_xyz_lines:
        if "HETATM" in p_xyz_line and ligname in p_xyz_line[17:21]:
            atom_name = p_xyz_line[12:16]
            if atom_name in atom_list:
                multi_ligand = True
            else:
                atom_list.append(atom_name)
                l_xyz_lines.append(p_xyz_line)
    if multi_ligand:
        logging.info("Warning: Found multiple ligand for this protein:%s"%proteinfile)
    #here change to save ligand even there are mutiple ligands
    f = open(ligandfile, "w")
    f.writelines(l_xyz_lines)
    f.close()
    return ligandfile

def get_center(ligand_pdb):
    xyz_lines = open(ligand_pdb,"r").readlines()
    multi_ligand = False
    atom_list = []
    x = y = z = 0
    for xyz_line in xyz_lines:
        if "HETATM" in xyz_line:
            #logging.debug("Check the get center of this protein: %s for this ligand: %s"%(protein_file, ligname))                                                                                                               
            atom_name = xyz_line[12:16]
            if atom_name in atom_list:
                multi_ligand = True
            else:
                atom_list.append(atom_name)
                try:
                    x += float(xyz_line[30:38])
                    y+= float(xyz_line[38:46])
                    z+= float(xyz_line[46:54])
                except:
                    logging.debug("Fatal error: Cannot find the XYZ coordinate for this ligand:%s"%ligand_pdb)
                    return False
    if not multi_ligand:
        lig_center = "%8.3f, %8.3f, %8.3f"%(x/len(atom_list), y/len(atom_list), z/len(atom_list))
        logging.debug("Ligand center for this case:%s is %s"%(ligand_pdb, lig_center))
        return lig_center
    else:
        logging.debug("Fatal error: Found multiple ligands in file:%s"%ligand_pdb)
        return False
        
def main_gendata (s3_result_path, path_2_ent, s4_result_path):
    os.chdir(s4_result_path)
    current_dir_layer_1 = os.getcwd()
    blastnfilterout = glob.glob("%s/*.txt"%s3_result_path)
    summary_file = "%s/summary.txt"%s3_result_path                         
    if summary_file in blastnfilterout:                                    
        blastnfilterout.remove(summary_file)
    problem_cases = []
    valid_cases = []
    all_cases = []
    for single_bfout in blastnfilterout:
        valid = False
        target_name = os.path.basename(single_bfout).split(".txt")[0]
        all_cases.append(target_name)
        if not os.path.isdir(target_name):
            os.mkdir(target_name)
        os.chdir(target_name)
        commands.getoutput("cp %s ."%single_bfout)
        try:
            info_dic = parse_txt(single_bfout)
        except:
            logging.info("Could not parse this blasternfilter outfile: %s"%(single_bfout))
            continue
        #check if the is a protein start with LMCSS
        if not "LMCSS" in info_dic:
            logging.info("For this query protein: %s, there is no protein sharing the LMCSS ligand with. Not able to generate Docking grid, pass for this case..."%target_name)
            os.chdir (current_dir_layer_1)
            continue
        elif len(info_dic["LMCSS"]) != 3:
            logging.info("For this query protein: %s, the LMCSS protein has wrong number of informations associate with this id..."%target_name)
            os.chdir(current_dir_layer_1)
            continue
        elif not "inchi" in info_dic:
            logging.info("For this query protein: %s, there is no inchi info for the ligand..."%target_name)
            os.chdir(current_dir_layer_1)
            continue
        else:
            logging.info("============Start to work in this query protein: %s============"%target_name)

        #get all ligand related files
        try:
            inchi = info_dic["inchi"]
            ligand_title = info_dic["ligand"]
        #probably will come back sliu
            generate_ligand(inchi, ligand_title)
        except:
            logging.info("Could not generate the ligand for this case: %s"%(target_name))
            os.chdir(current_dir_layer_1)                     
            continue
        #copy all ent file here and rename it
        query_pro = info_dic["query"]
        LMCSS_pro_id = info_dic["LMCSS"][0]
        LMCSS_pro_ligand = info_dic["LMCSS"][1]
        LMCSS_mcss_chain = info_dic["LMCSS"][2]
        LMCSS_pdb_folder_name = LMCSS_pro_id[1:3]
        LMCSS_ent_file = "pdb" + LMCSS_pro_id  + ".ent"
        LMCSS_pdbloc = os.path.join(path_2_ent, LMCSS_pdb_folder_name, LMCSS_ent_file)
        if not os.path.isfile(LMCSS_pdbloc):            
            logging.info("Unable to find the ent file associate with the LMCSS pdb: %s at location %s"%(LMCSS_pro_id, LMCSS_pdbloc))
            os.chdir(current_dic_layer_1)
            continue
        else:
            #TC = target candidate                                
            TC_id = "%s_%s"%(query_pro, LMCSS_pro_id)             
            LMCSS_protein_name = "LMCSS-%s-%s.pdb"%(TC_id, LMCSS_pro_ligand)
            LMCSS_ligand_name = "LMCSS-%s-%s-lig.pdb"%(TC_id, LMCSS_pro_ligand)
            #try to extract the LMCSS ligand from the LMCSS protein
            commands.getoutput("cp %s %s"%(LMCSS_pdbloc,LMCSS_protein_name))
            #extract chain where contain the LMCSS mcss
            split_chain(LMCSS_protein_name, LMCSS_protein_name, LMCSS_pro_id, LMCSS_mcss_chain)
            if not pull_ligand_out (LMCSS_protein_name, LMCSS_pro_ligand, LMCSS_ligand_name):
                os.chdir(current_dir_layer_1)
                continue
                
            logging.info("Succsessfully generate this protein:%s"%LMCSS_protein_name)
            ligand_center = get_center (LMCSS_ligand_name)
            if not ligand_center:
                logging.info("Unable to find the center of the ligand for the LMCSS candidate pdb: %s"%(pot_target_id))
                os.chdir(current_dir_layer_1)
                continue
            else:
                with open("center.txt" , "w") as center_file:
                    center_file.writelines(ligand_center)
            for rest_protein in ("SMCSS", "hiResHolo", "hiResApo", "hiTanimoto"):
                if rest_protein in info_dic:
                    if len(info_dic[rest_protein]) == 3:
                        rest_protein_id = info_dic[rest_protein][0] 
                        rest_ligand_id = info_dic[rest_protein][1]
                        rest_ligand_chain = info_dic[rest_protein][2]
                    else:
                        rest_protein_id = info_dic[rest_protein]
                        rest_ligand_id = False
                    rest_protein_folder_name = rest_protein_id[1:3]
                    rest_ent_file = "pdb" + rest_protein_id  + ".ent"       
                    rest_pdbloc = os.path.join(path_2_ent, rest_protein_folder_name, rest_ent_file)
                    if not os.path.isfile(rest_pdbloc):       
                        logging.info("Unable to find the ent file associate with the %s pdb with ID: %s at location %s"%(rest_protein, rest_protein_id, rest_pdbloc))
                        os.chdir(current_dic_layer_1)
                        continue
                    else:
                        TC_id_rest = "%s_%s"%(query_pro, rest_protein_id)
                        if rest_ligand_id:
                            rest_protein_name = "%s-%s-%s.pdb"%(rest_protein, TC_id_rest, rest_ligand_id)
                            commands.getoutput("cp %s %s"%(rest_pdbloc, rest_protein_name))
                            #extract ligand chain from proteins with ligand, original just do this in LMCSS, 0908 sliu 
                            split_chain(rest_protein_name, rest_protein_name, rest_protein_id, rest_ligand_chain)
                        else:
                            rest_protein_name = "%s-%s.pdb"%(rest_protein, TC_id_rest)
                            commands.getoutput("cp %s %s"%(rest_pdbloc, rest_protein_name))
                        #align all rest proteins onto the LMCSS protein
                        
                        try:
                            do_alignment = align_proteins (LMCSS_protein_name, rest_protein_name, rest_protein_name)
                        except:
                            logging.info("The alignment could not be done for this protein:%s"%(rest_protein_name))
                            os.chdir(current_dic_layer_1)
                            continue

                            #remove the "rot-%s"%LMCSS_protein_name
                        if os.path.isfile("rot-%s"%LMCSS_protein_name):
                            commands.getoutput("rm rot-%s"%LMCSS_protein_name) 
                        logging.info("Succsessfully generate this protein:%s"%(rest_protein_name))
                        #here we got the valid aligned structures
                        valid = True
        os.chdir(current_dir_layer_1)        
        #here remove all the folder which don't have full information because of all errors which showed in the final.log
        if valid:
            valid_cases.append(target_name)
    for case in all_cases:
        if case not in valid_cases:
            problem_cases.append(case)
            if not os.path.isdir("error_container"):
                os.mkdir("error_container")
            commands.getoutput("mv %s error_container"%case)
    logging.info("Finish generating the challenge data. The problematic cases are: %s"%problem_cases)

if ("__main__") == (__name__):
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-p", "--pdbdb", metavar = "PATH", help = "PDB DATABANK which we will dock into")
    parser.add_option("-c", "--candidatedir", metavar="PATH", help = "PATH where we could find the stage 3 output")
    parser.add_option("-o", "--outdir", metavar = "PATH", help = "PATH where we run stage 4")
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )
    (opt, args) = parser.parse_args()
    pdb_location = opt.pdbdb
    stage_3_result = opt.candidatedir
    stage_4_result = opt.outdir
    running_dir = os.getcwd()
    main_gendata(stage_3_result, pdb_location, stage_4_result)
    log_file_path = os.path.join(running_dir, 'final.log')
    commands.getoutput("mv %s %s"%(log_file_path,stage_4_result))



# -*- coding: utf-8 -*-

__author__ = 'churas'

import os
import logging
import re

from d3r.celpp import util
from d3r.celpp.task import D3RTask
from d3r.celpp.makeblastdb import MakeBlastDBTask
from d3r.celpp.dataimport import DataImportTask


logger = logging.getLogger(__name__)


class BlastNFilterSummary:
    """Represents summary of a BlastNFilterTask invocation
    """
    def __init__(self, path):

        if path is None:
            logger.error('Path passed into constructor is None type')
            self._path = ''
        else:
            self._path = path

        self._complexes = 0
        self._dockable_complexes = 0
        self._dockable_monomers = 0
        self._targets_found = 0
        self._parse_summary_file()

    def _parse_summary_file(self):
        """Parses summary file to obtain needed fields in object

           WARNING: This method assumes a well formed summary.txt file and
           if its not then output maybe wonky
        """
        if not os.path.isdir(self._path):
            logger.error(self._path + ' is not a directory')
            return

        # open summary.txt file and extract
        # complexes, dockable complexes, dockable monomers and Targets found
        # below is example summary.txt file with parts omitted
        #
        # INPUT SUMMARY
        #   entries:                             135
        #   complexes:                            83
        #   dockable complexes:                   46
        #   monomers:                             90
        #   dockable monomers:                    38
        #   multimers:                            45
        #   dockable multimers:                    8
        # .
        # . <THERE IS OTHER DATA HERE>
        # .
        # OUTPUT SUMMARY
        #   Targets found:                        33
        # .
        # .
        summary_txt = os.path.join(self._path, BlastNFilterTask.SUMMARY_TXT)
        if not os.path.isfile(summary_txt):
            logger.error('No ' + summary_txt + ' file found')
            return

        complexes_pat = re.compile("^ +complexes: +")
        dock_complex_pat = re.compile("^ +dockable complexes: +")
        dock_mon_pat = re.compile("^ +dockable monomers: +")
        targets_pat = re.compile("^ +Targets found: +")

        logger.debug('Parsing file ' + summary_txt)

        f = open(summary_txt, 'r')
        for line in f:
            if re.match(complexes_pat, line):
                self.set_complexes(re.sub("^.*: +", "", line.rstrip()))
            elif re.match(dock_complex_pat, line):
                self.set_dockable_complexes(re.sub("^.*: +", "",
                                                   line.rstrip()))
            elif re.match(dock_mon_pat, line):
                self.set_dockable_monomers(re.sub("^.*: +", "", line.rstrip()))
            elif re.match(targets_pat, line):
                self.set_targets_found(re.sub("^.*: +", "", line.rstrip()))
        f.close()

    def get_complexes(self):
        return self._complexes

    def set_complexes(self, val):
        self._complexes = val

    def get_dockable_complexes(self):
        return self._dockable_complexes

    def set_dockable_complexes(self, val):
        self._dockable_complexes = val

    def get_dockable_monomers(self):
        return self._dockable_monomers

    def set_dockable_monomers(self, val):
        self._dockable_monomers = val

    def get_targets_found(self):
        return self._targets_found

    def set_targets_found(self, val):
        self._targets_found = val

    def get_week_number(self):
        """Parses path for week number
           :returns: week number as string
        """
        return util.get_celpp_week_number_from_path(
            os.path.dirname(self._path))

    def get_year(self):
        """Parses year from path passed into constructor
           :returns: year as string
        """
        return util.get_celpp_year_from_path(self._path)

    def get_csv(self):
        """Returns comma separated string of values parsed from summary.txt

           :returns: Week #, Year, # complexes, # dockable complexes,
                   # dockable monomers, # targets found
        """
        return (self.get_week_number() + ',' + self.get_year() + ',' +
                str(self.get_complexes()) + ',' +
                str(self.get_dockable_complexes()) +
                ',' + str(self.get_dockable_monomers()) + ',' +
                str(self.get_targets_found()))


class BlastNFilterTask(D3RTask):
    """Performs blast and filter of sequences

    """
    SUMMARY_TXT = "summary.txt"
    DOCKABLE_XSLX = "dockable.xlsx"
    BLASTNFILTER_LOG = "blastnfilter.log"
    TASK_NAME = 'blastnfilter'

    def __init__(self, path, args):
        super(BlastNFilterTask, self).__init__(path, args)
        self.set_name(BlastNFilterTask.TASK_NAME)

        # set the stage to be 1 higher then Data Import Stage
        dataimport = DataImportTask(path, args)
        self.set_stage(dataimport.get_stage() + 1)
        self.set_status(D3RTask.UNKNOWN_STATUS)

    def get_blastnfilter_summary(self):
        """Factory method that Generates BlastNFilterSummary object

           by parsing data from this object
        """
        return BlastNFilterSummary(self.get_dir())

    def get_blastnfilter_summary_file(self):
        """Returns path to summary.txt file
        """
        return os.path.join(self.get_dir(), BlastNFilterTask.SUMMARY_TXT)

    def _parse_blastnfilter_output_for_hit_stats(self):
        """Examines output directory of blastnfilter.py for stats on run

           This method looks at the output directory and counts the number
           of .csv files found.  Each .csv file corresponds to a target.
           Within each .csv file the entries under Test Information correspond
           to candidates.  This method will output number of candidates for
           that target by calling get_candidate_count_from_csv method
        """

        txt_list = self.get_txt_files()
        summary_out = '\nOutput from ' + BlastNFilterTask.SUMMARY_TXT + '\n'
        summary_txt = os.path.join(self.get_dir(),
                                   BlastNFilterTask.SUMMARY_TXT)
        try:
            if os.path.isfile(summary_txt):
                f = open(summary_txt, 'r')
                for entry in f:
                    summary_out = (summary_out + entry)
                f.close()
        except:
            logger.warning('Error reading ' + summary_txt + ' file')

        return '\n# txt files found: ' + str(len(txt_list)) +\
               '\n' + summary_out

    def get_txt_files(self, addfullpath=False):
        """ Gets txt files in task directory (just the names) skiping summary.txt
        :return:list of txt file names
        """
        out_dir = self.get_dir()
        txt_list = []
        try:
            for entry in os.listdir(out_dir):
                if entry == BlastNFilterTask.SUMMARY_TXT:
                    continue

                if entry.endswith('.txt'):
                    if addfullpath is True:
                        txt_list.append(os.path.join(out_dir, entry))
                    else:
                        txt_list.append(entry)
        except OSError:
            logger.warning('Caught exception trying to look for .txt files')

        return txt_list

    def get_uploadable_files(self):
        """Returns a list of files that can be uploaded to remote server

           Provides a way for a D3RTask to define what files should
           uploaded to remote server by any D3RTaskUploader.  The default
           implementation returns an empty list
           :returns: list of files that can be uploaded.  The files should
           have full paths
        """

        # get the stderr/stdout files
        file_list = super(BlastNFilterTask, self).get_uploadable_files()

        out_dir = self.get_dir()
        # add txt files
        file_list.extend(self.get_txt_files(True))

        summary_file = os.path.join(out_dir,
                                    BlastNFilterTask.SUMMARY_TXT)
        if os.path.isfile(summary_file):
            file_list.append(summary_file)

        dockable_file = os.path.join(out_dir,
                                     BlastNFilterTask.DOCKABLE_XSLX)

        if os.path.isfile(dockable_file):
            file_list.append(dockable_file)

        bnf_log_file = os.path.join(out_dir, BlastNFilterTask.BLASTNFILTER_LOG)

        if os.path.isfile(bnf_log_file):
            file_list.append(bnf_log_file)
        return file_list

    def can_run(self):
        """Determines if task can actually run

           This method first verifies the `MakeBlastDBTask` and
           `DataImportTask` task have
           `D3RTask.COMPLETE_STATUS` for status.  The method then
           verifies a `BlastNFilterTask` does not already exist.
             If above is not true then self.set_error() is set
             with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        # check blast
        make_blastdb = MakeBlastDBTask(self._path, self._args)
        make_blastdb.update_status_from_filesystem()
        if make_blastdb.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + ' task ' +
                        'because ' + make_blastdb.get_name() + ' task' +
                        'has a status of ' + make_blastdb.get_status())
            self.set_error(make_blastdb.get_name() + ' task has ' +
                           make_blastdb.get_status() + ' status')
            return False

        # check data import
        data_import = DataImportTask(self._path, self._args)
        data_import.update_status_from_filesystem()
        if data_import.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + ' task ' +
                        'because ' + data_import.get_name() + ' task' +
                        'has a status of ' + data_import.get_status())
            self.set_error(data_import.get_name() + ' task has ' +
                           data_import.get_status() + ' status')
            return False

        # check blast is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Runs blastnfilter task after verifying dataimport was good

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes blastnfilter script and
           postanalysis script.  Upon completion results are
           analyzed and success or error status is set
           appropriately and D3RTask.end is invoked
           """
        super(BlastNFilterTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        data_import = DataImportTask(self._path, self._args)

        make_blastdb = MakeBlastDBTask(self._path, self._args)

        try:
            loglevel = self.get_args().loglevel
        except AttributeError:
            logger.debug('No log level set in arguments using WARNING')
            loglevel = 'WARNING'

        try:
            blastnfiltertimeout = self.get_args().blastnfiltertimeout
            logger.debug('blastnfilter timeout set to ' +
                         str(blastnfiltertimeout))
        except AttributeError:
            blastnfiltertimeout = None

        # verify sequence.tsv file exists on filesystem.
        # if not fall back to oldsequence.tsv file
        sequencetsv = data_import.get_sequence_tsv()
        if not os.path.isfile(sequencetsv):
            logger.warning(sequencetsv + ' file not found. falling '
                                         'back to old file')
            self.append_to_email_log('\n ' + sequencetsv + ' file not found ' +
                                     'falling back to ' +
                                     data_import.get_oldsequence_tsv() + '\n')
            sequencetsv = data_import.get_oldsequence_tsv()

        cmd_to_run = (self.get_args().blastnfilter + ' --nonpolymertsv ' +
                      data_import.get_nonpolymer_tsv() +
                      ' --sequencetsv ' +
                      sequencetsv +
                      ' --pdbblastdb ' +
                      make_blastdb.get_dir() +
                      ' --compinchi ' +
                      data_import.get_components_inchi_file() +
                      ' --crystalpH ' +
                      data_import.get_crystalph_tsv() +
                      ' --pdbdb ' +
                      self.get_args().pdbdb +
                      ' --log ' +
                      loglevel +
                      ' --outdir ' + self.get_dir())

        blastnfilter_name = os.path.basename(self.get_args().blastnfilter)

        self.run_external_command(blastnfilter_name,
                                  cmd_to_run, False,
                                  timeout=blastnfiltertimeout)

        self.set_status(D3RTask.COMPLETE_STATUS)

        cmd_to_run = (self.get_args().postanalysis + ' --compinchi ' +
                      data_import.get_components_inchi_file() + ' ' +
                      self.get_dir())

        postanalysis_name = os.path.basename(self.get_args().postanalysis)

        self.run_external_command(postanalysis_name,
                                  cmd_to_run, False,
                                  timeout=blastnfiltertimeout)

        try:
            # examine output to get candidate hit count DR-12
            hit_stats = self._parse_blastnfilter_output_for_hit_stats()
            if hit_stats is not None:
                self.append_to_email_log(hit_stats)
        except Exception:
            logger.exception("Error caught exception")

        # assess the result
        self.end()



__author__ = 'robswift'

import os
import logging
from StringIO import StringIO
from Bio.Blast.Applications import NcbiblastpCommandline
from Bio.Blast import NCBIXML
from Bio.Alphabet import IUPAC
from Bio.Seq import Seq
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from d3r.blast.base import Base
from d3r.blast.hit import Hit
from d3r.blast.ligand import Ligand

logger = logging.getLogger(__name__)


class Query(Base):
    """
    This class contains information about the macromolecular structures uniquely identified by a wwPDB in the
    new_release_structure_sequence.tsv and new_release_structure_nonpolymer.tsv files. The Target class contains
    a list of d3r.Test objects that are instantiated following a BLASTP of the Target sequence.
    Information is included to facilitate filtering:
      (i) self.contains_largest_mcss: Set to True if the maximum common substructure between the dockable ligand of the
      Target object, self.dock, and one of the contained d3r.blast.Test objects dockable ligands is largest among a
      set of Target objects.
      (ii) self.contains_smallest_mcss: Set to True if the maximum common substructure between the dockable ligand of
      the Target object, self.dock, and one of the contained d3r.blast.Test objects dockable ligands is smallest among
      a set of Target objects.
      (iii) self.inchi_error: Set to True if the Target object contains a dockable ligand and an error is produced when
      attempting to convert the InChI to a rdkit Mol object. Set to False if the Target object contains a dockable
      ligand and the InChI was successfully converted. Set to None if the Target object is Apo, or contains no dockable
      ligand.
    """
    def __init__(self):
        super(Query, self).__init__()              # inherit superclass constructor
        self.hits = []                            # [test object, test object, ... ]
        self.hit_membership = {}                  # {'PDB_ID' : index (int)} where PDB_ID is a lowercase string and
                                                   # index is the index of the corresponding PDB in test_list
        self.inchi_error = None                    # None, True, or False.
        self.exp_ph = None                         # pH @ which the structure was solved. Only valid for xtal structs.

    def set_ligand(self, resname, inchi, label):
        """
        Creates a d3r.blast.Ligand object entry for the ligand. If the ligand is dockable (not in in the
        d3r.filter.FilteringSets do_not_call set), the Ligand object is appended to the dock list, and the dock_count
        and ligand_count attributes are each incremented. If ligand is in the do_not_call filtering set, then the
        ligand is added to the do_not_call list, and the ligand_count attribute is incremented. Note that, no effort is
        made to check if the input ligand is unique or has not been added before. In the case of a dockable ligand, an
        attempt is made to convert the InChI to a rd mol object. If the conversion is successful, then the Target
        inchi_error attribute is set to False. If the conversion is unsuccessful, then the the inchi_error attribute is
        set to True. Non-dockable ligands are not converted to rd mol objects.
        :param resname: (string) the PDB resname of the input ligand
        :param inchi: (string) the inchi string, which uniquely identifies the ligand
        :param label: (string) set to either 'dock' or 'do_not_call'
        """
        logger.debug('In set_ligand()')
        ligand = Ligand(resname, inchi)
        #modified by sliu 08/08, set the ligand size, rot bond etc
        if label == 'dock':
            if ligand.set_rd_mol_from_inchi():
                self.inchi_error = False
            else:
                self.inchi_error = True
            ligand.set_size()
            ligand.set_heavy_size()
            ligand.set_rot()
            self.dock.append(ligand)
            self.dock_count += 1
            self.ligand_count += 1
        elif label == 'do_not_call':
            self.do_not_call.append(ligand)
            self.ligand_count += 1
        else:
            pass

    def set_sequence(self, chain_id, seq):
        """
        Converts an input FASTA sequence to a Bio.SeqRecord with the id field set to the input integer chain id. Each
        unique sequence receives a unique integer chain id, so chain ids will be numbered 1 through N, where N is the
        count of the last unique chain of the protein. Additionally, each time a sequence is added, the
        sequence_count attribute of the Target object will be incremented. Note that no effort is made to check whether or
        not the input sequence is unique, i.e. that it hasn't been added before.
        :param chain_id: (an integer cast
        :param seq:  a FASTA sequence
        """
        id = '{pdb_id}_{chain_id}'.format(pdb_id = self.pdb_id, chain_id = chain_id)
        self.sequences.append(SeqRecord(Seq(seq, IUPAC.protein), id = id))
        self.sequence_count += 1

    def run_blast(self, pdb_db, out_dir):
        """
        Runs BLASTP on the sequence(s) contained in the Query instance. If the chain of a sequence has not been
        assigned to the Query instance, then False is returned. Otherwise, the sequence returns a
        Bio.blast.Record object. If the Query instance is a multimer, the alignment objects in the list,
        Bio.blast.Record.alignments are the intersection of the alignment objects, or BLASTP hits that result from
        running BLASTP on each of the sequences of each of the unique chains that constitutes the multimer. If the
        Target instance is a monomer, the alignment objects in the list Bio.blast.Record.alignments is simply the list
        of BLASTP hits that result from running BLASTP on the single unique sequence that makes up the monomer.
        :param pdb_db: (string) The absolute path to a BLASTP database
        :param out_dir: (string) The absolute path to the output directory. A fasta file is writen here prior to running
        BLASTP and is deleted after the BLASTP search is completed.
        :return: Bio.blast.Record object
        """
        logger.debug('In run_blast()')
        if self.sequence_count == 0:
            logger.error("Fasta sequences have not been added to the " +
                         "Target.sequences list, and BLAST cannot be run")
            return False
        elif self.sequence_count == 1:
            logger.debug('Running blast_monomer')
            record = self.blast_monomer(pdb_db, out_dir)
            return record
        elif self.sequence_count > 1:
            logger.debug('Running blast_multimer')
            records = self.blast_multimer(pdb_db, out_dir)
            return records

    def blast_monomer(self, pdb_db, out_dir):
        """
        Runs a BLASTP search of sequences in the wwPDB for a single monomer chain
        :param pdb_db: (string) The absolute path to a BLASTP database
        :param out_dir: (string) The absolute path to the output directory. A fasta file is writen here prior to running
        :return: Bio.blast.Record
        """
        records = []
        try:
            logger.debug('There are ' +
                         str(len(self.sequences)) + ' to ' +
                         'analyze but this code will only analyze '
                         'the first sequence???')
        except:
            logger.exception('Caught exception logging debug information')
        
        for sequence in self.sequences:
            fasta = self.write_fasta(sequence, out_dir)
            if fasta:
                logger.debug('Starting blastp of ' + fasta)
                record = self.blastp(fasta, pdb_db)
                logger.debug('Removing file ' + fasta)
                os.remove(fasta)
                if record:
                    records.append(record)
                return records

    def blast_multimer(self, pdb_db, out_dir):
        """
        Runs a BLASTP search of sequences in the wwPDB for each unique chain in a multimer.
        :param pdb_db: (string) The absolute path to a BLASTP database
        :param out_dir: (string) The absolute path to the output directory. A fasta file is writen here prior to running
        :return: Bio.blast.Record or None
        """
        records = []
        try:
            logger.debug('There are ' + str(len(self.sequences)) + ' to analyze')
        except:
            logger.exception('Caught exception logging debug information')
        for sequence in self.sequences:
            fasta = self.write_fasta(sequence, out_dir)
            if fasta:
                logger.debug('Starting blastp of ' + fasta)
                records.append(self.blastp(fasta, pdb_db))
                logger.debug('Removing ' + fasta + ' file')
                os.remove(fasta)
        if records:
            logger.debug('Getting intersection of records')
            self.get_intersection(records)
            return records

    def get_intersection(self, records):
        """
        Returns a Bio.blast record object, whose alignment objects are the intersection of the alignments stored in
        each of the Bio.blast record objects contained in the input records list.
        :param records: a list of Bio.blast.Record objects
        :return: record, a Bio.blast.Record object
        """
        ids=[]
        try:
            logger.debug('In get_intersection() examining ' +
                         str(len(records)) + ' records')
        except:
            logger.exception('caught exception printing log message')
        for record in records:
            ids.append(set([self.get_pdb_id_from_alignment(a) for a in record.alignments]))
        id_intersection = set.intersection(*ids)
        self.clean_alignments(records, id_intersection)

    def clean_alignments(self, records, id_intersection):
        """
        Removes all alignments from the input Bio.blast.Record object whose sequences correspond to wwPDB IDs not in the
        input intersection set. It's assumed that the wwPDB IDs of the input Bio.blast.Record.alignments list can form
        a superset of the input intersection set.
        :param record: a Bio.blast.Record object
        :param id_intersection: (set) a set of wwPDB ids that represent the intersection of two or more BLASTP results.
        :return:
        """
        for record in records:
            delete = []
            for index in range(len(record.alignments)):
                if self.get_pdb_id_from_alignment(record.alignments[index]) not in id_intersection:
                    delete.append(index)
            delete.sort(reverse = True)
            for index in delete:
                del record.alignments[index]

    def blastp(self, fasta, pdb_db):
        """
        Runs BLASTP locally on a input fasta file and specified BLASTP database
        :param fasta: The absolute path to a FASTA file
        :param pdb_db: A BLASTP database
        :return: Bio.blast.Record object
        """
        logger.debug('Running blastp ' + fasta)
        cline = NcbiblastpCommandline(cmd='blastp', query=fasta, db=pdb_db, evalue=0.001, outfmt=5)
        std_out, std_err = cline()
        blast_records = NCBIXML.parse(StringIO(std_out))
        record = next(blast_records)
        return record

    def write_fasta(self, sequence, out_dir):
        """
        Writes the input BioPython sequence records to a fasta file in the
        specified directory. If the fasta file was
        written successfully, the absolute path to the fasta file is
        returned. Otherwise False is returned.
        :sequence: (Bio.SeqRecord)
        :param outdir: (string) The absolute path of the directory
        where the fasta file will be written
        return: the absolute path to the fasta file upon success or
        False upon failure
        """
        fasta = os.path.join(os.path.abspath(out_dir), '_'.join((self.pdb_id, sequence.id)) + '.fasta')
        logger.debug('Writing fasta file: ' + fasta)
        try:
            SeqIO.write(sequence, fasta, "fasta")
            return fasta
        except:
            logger.exception('Caught exception')
            return False
        return False

    def get_pdb_id_from_alignment(self, alignment):
        """
        Returns a string of the four letter PDB ID found in alignment
        :param alignment:
        :return: 'pdb id'
        """
        pdb_chain_id = alignment.hit_def.encode('ascii').split()[0]
        pdb_id = pdb_chain_id.split('_')[0].lower()
        return pdb_id

    def get_chain_id_from_alignment(self, alignment):
        """
        Returns a string of the chain id, e.g. 'A', 'B', etc.
        :param alignment:
        :return:
        """
        pdb_chain_id = alignment.hit_def.encode('ascii').split()[0]
        chain = pdb_chain_id.split('_')[1].upper()
        return chain

    def sort_by_resolution(self):
        """
        If a test list exists, and each test object has a resolution value, the list is sorted from highest resolution
        (lowest value) to lowest resolution (highest value)
        """
        self.hits.sort(key=lambda x: x.get_resolution())

    def set_hits(self, records):
        """
        Populate self.test_list with a list of hit objects. Each hit object corresponds to a BLASTP hit stored in one
        of the alignment objects found in the input record.alignments list and contains the following information about
        the structural hit: wwPDB id, % coverage (alignment length / query length), % identity (number of identical
        amino acids in the alignment / hit length), experimental structural determination method, resolution,
        :param record (Bio.blast.Record object)
        """
        # do something to set chain count and include those chains not in alignments
        #### add stuff to make this work...move this stuff inside Query
        logger.debug('Found ' + str(len(records)) + ' of hits')
        for record in records:
            logger.debug('Found ' + str(len(record.alignments)) + ' alignments for record')
            for alignment in record.alignments:
                pdb_id = self.get_pdb_id_from_alignment(alignment)
                #print "\tCheck the alignment:%s, and pdb ID:%s "%(alignment, pdb_id)
                chain_id = self.get_chain_id_from_alignment(alignment)
                #print "\tCheck the chain ID", chain_id
                if pdb_id in self.hit_membership.keys():
                    self.hits[self.hit_membership[pdb_id]].set_sequence(pdb_id, chain_id, record, alignment)
                    #sliu 0808 add chain info in query
                    self.hits[self.hit_membership[pdb_id]].set_ligands(chain_id)
                else:
                    hit = Hit()
                    hit.pdb_id = pdb_id
                    if not hit.read_pdb():
                        continue
                    hit.set_resolution()
                    hit.set_expt_method()
                    #sliu 01092017 fix the bug to skip the case where set sequence is not finished
                    if hit.set_sequence(pdb_id, chain_id, record, alignment):
                        hit.set_ligands(chain_id)
                    #print "Inside query set hits> set ligands check after set ligands"
                    #raw_input()
                        self.hits.append(hit)
                        self.hit_membership[pdb_id] = len(self.hits) - 1

    def fill_sequences(self):
        """
        Iterates over each hit and calls the fill_sequence method. The result is that for each hit, chains that are
        present in the pdb that weren't picked up by blast are added. For example, these can include nucleic acid
        sequences that weren't co-crystallized with the protein. This method should only be called after the set_hits()
        method is called and all the hit sequences were initiated.
        """
        for hit in self.hits:
            hit.fill_sequence()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_celppreports
----------------------------------

Tests for `celppreports` module.
"""

import unittest
import tempfile
import logging
import os
import os.path
import shutil

from d3r import celppreports
from d3r.celpp.task import D3RParameters
from d3r.celpp.blastnfilter import BlastNFilterTask


class TestCelppReports(unittest.TestCase):

    def setUp(self):
        pass

    def test_setup_logging(self):
        theargs = D3RParameters()

        theargs.loglevel = 'DEBUG'
        celppreports._setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.blastnfilter')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.dataimport')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.glide')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.makeblastdb')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.proteinligprep')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.evaluation')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.task')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.util')
                         .getEffectiveLevel(), logging.DEBUG)
        self.assertEqual(logging.getLogger('d3r.celpp.filetransfer')
                         .getEffectiveLevel(), logging.DEBUG)

        theargs.loglevel = 'INFO'
        celppreports._setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.blastnfilter')
                         .getEffectiveLevel(), logging.INFO)

        theargs.loglevel = 'WARNING'
        celppreports._setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.blastnfilter')
                         .getEffectiveLevel(), logging.WARNING)

        theargs.loglevel = 'ERROR'
        celppreports._setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.blastnfilter')
                         .getEffectiveLevel(), logging.ERROR)

        theargs.loglevel = 'CRITICAL'
        celppreports._setup_logging(theargs)
        self.assertEqual(logging.getLogger('d3r.celpp.blastnfilter')
                         .getEffectiveLevel(), logging.CRITICAL)

    def test_parse_arguments(self):
        theargs = ['--outdir', '/outdir', '--log', 'INFO', 'celppdir']
        result = celppreports._parse_arguments('hi', theargs)
        self.assertEqual(result.outdir, '/outdir')
        self.assertEqual(result.loglevel, 'INFO')
        self.assertEqual(result.celppdir, 'celppdir')

    def test_generate_reports_celppdir_does_not_exist(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()
            theargs.celppdir = os.path.join(temp_dir, 'doesnotexist')
            try:
                celppreports.generate_reports(theargs)
                self.assertEqual('Expected exception')
            except Exception:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_reports_outdir_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()

            theargs.celppdir = temp_dir
            try:
                celppreports.generate_reports(theargs)
                self.assertEqual('Expected exception')
            except Exception:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_reports_outdir_set_to_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()

            theargs.celppdir = temp_dir
            theargs.outdir = None
            try:
                celppreports.generate_reports(theargs)
                self.assertEqual('Expected exception')
            except Exception:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_reports_outdir_needs_to_be_created_no_celpp_data(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()

            theargs.celppdir = temp_dir
            outdir = os.path.join(temp_dir, 'outdir')
            theargs.outdir = outdir

            celppreports.generate_reports(theargs)
            self.assertEqual(os.path.isdir(outdir), True)
            csv_file = os.path.join(outdir, 'blastnfilter.summary.csv')
            self.assertEqual(os.path.isfile(csv_file), True)

        finally:
            shutil.rmtree(temp_dir)

    def test_generate_reports_with_one_week_in_one_year(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = D3RParameters()

            theargs.celppdir = temp_dir
            outdir = os.path.join(temp_dir, 'outdir')
            theargs.outdir = outdir

            yeardir = os.path.join(temp_dir, '2016')
            os.mkdir(yeardir)
            weekdir = os.path.join(yeardir, 'dataset.week.10')
            os.mkdir(weekdir)
            blast = BlastNFilterTask(temp_dir, theargs)
            blastdir = os.path.join(weekdir, blast.get_dir_name())
            os.mkdir(blastdir)

            # create dummary summary.txt file

            f = open(os.path.join(blastdir,
                                  BlastNFilterTask.SUMMARY_TXT), 'w')
            f.write('INPUT SUMMARY\n')
            f.write('  entries:                             221\n')
            f.write('  complexes:                           178\n')
            f.write('  dockable complexes:                   95\n')
            f.write('  monomers:                            145\n')
            f.write('  dockable monomers:                    71\n')
            f.write('  multimers:                            76\n')
            f.write('  dockable multimers:                   24\n\n')

            f.write('FILTERING CRITERIA\n')
            f.write('  No. of query sequences           <=    1\n')
            f.write('  No. of dockable ligands           =    1\n')
            f.write('  Percent identity                 >=    0.95\n')
            f.write('  Percent Coverage                 >=    0.9\n')
            f.write('  No. of hit sequences             <=    4\n')
            f.write('  Structure determination method:        '
                    '  x-ray diffraction\n\n')

            f.write('OUTPUT SUMMARY\n')
            f.write('  Targets found:                        67\n')
            f.write('  Target: 5fz7|Sequences: 1|Hits: 94|Candidates: 17|'
                    'Elected:4|PDBids: 5fz7,5fyz,5a3p,5a1f\n')

            f.flush()
            f.close()

            celppreports.generate_reports(theargs)
            self.assertEqual(os.path.isdir(outdir), True)
            csv_file = os.path.join(outdir, 'blastnfilter.summary.csv')
            self.assertEqual(os.path.isfile(csv_file), True)

            # check csv file
            f = open(csv_file, 'r')
            header = f.readline()
            self.assertEqual(header, 'Week #, Year, Complexes, Dockable '
                                     'complexes, Dockable monomers, '
                                     'Targets Found\n')
            data = f.readline()
            self.assertEqual(data, '10,2016,178,95,71,67\n')

        finally:
            shutil.rmtree(temp_dir)

    def test_main_where_generate_reports_raises_error(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = ['prog', temp_dir]
            self.assertEqual(celppreports.main(theargs), 2)
        finally:
            shutil.rmtree(temp_dir)

    def test_main_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            theargs = ['prog', '--outdir', os.path.join(temp_dir, 'foo'),
                       temp_dir]
            self.assertEqual(celppreports.main(theargs), 0)
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()
